# src\file_conversor\backend\pypdf_backend.py

"""
This module provides functionalities for handling PDF files using ``pypdf`` backend.
"""

from pypdf import PdfReader, PdfWriter
from pypdf.constants import UserAccessPermissions

from enum import Enum
from pathlib import Path

from typing import Any, Callable, Iterable, Sequence

# user-provided imports
from file_conversor.config import Log

from file_conversor.utils.formatters import normalize_degree

from file_conversor.backend.abstract_backend import AbstractBackend

LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class PyPDFBackend(AbstractBackend):
    """
    A class that provides an interface for handling PDF files using ``pypdf``.
    """

    SUPPORTED_IN_FORMATS = {
        "pdf": {},
    }
    SUPPORTED_OUT_FORMATS = {
        "pdf": {},
    }
    EXTERNAL_DEPENDENCIES = set([])

    class EncryptionAlgorithm(Enum):
        """PDF encryption algorithms."""

        AES_256 = "AES-256"
        AES_256_R5 = "AES-256-R5"
        AES_128 = "AES-128"
        RC4_128 = "RC4-128"
        RC4_40 = "RC4-40"

        @classmethod
        def get_dict(cls):
            """Get dictionary of encryption algorithms."""
            return {
                "AES-256": PyPDFBackend.EncryptionAlgorithm.AES_256,
                "AES-256-R5": PyPDFBackend.EncryptionAlgorithm.AES_256_R5,
                "AES-128": PyPDFBackend.EncryptionAlgorithm.AES_128,
                "RC4-128": PyPDFBackend.EncryptionAlgorithm.RC4_128,
                "RC4-40": PyPDFBackend.EncryptionAlgorithm.RC4_40,
            }

        @classmethod
        def from_str(cls, name: str):
            """Get encryption algorithm from string."""
            alg_dict = cls.get_dict()
            if name not in alg_dict:
                raise ValueError(f"Encryption algorithm '{name}' is invalid. Valid options are: {', '.join(alg_dict.keys())}.")
            return alg_dict[name]

    @staticmethod
    def len(
            input_file: str | Path,
    ) -> int:
        """
        Get number of pages of input file.

        :param input_file: Input PDF file.

        :return: Number of pages.

        :raises FileNotFoundError: if input file not found
        """
        PyPDFBackend.check_file_exists(input_file)

        with PdfReader(input_file) as reader:
            return len(reader.pages)

    def __init__(
        self,
        verbose: bool = False,
    ):
        """
        Initialize the ``pypdf`` backend

        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__()
        self._verbose = verbose

    def merge(self,
              output_file: str | Path,
              input_files: Sequence[str | Path],
              password: str | None = None,
              progress_callback: Callable[[float], Any] | None = None,
              ):
        """
        Merge input files into an output.

        :param output_file: Output PDF file.
        :param input_files: Input PDF files. 
        :param password: Decryption password. Defaults to None.
        :param progress_callback: Progress callback (0-100). Defaults to None.

        :raises FileNotFoundError: if input file not found.
        """
        output_file = Path(output_file).with_suffix(".pdf")
        completed = 0.0
        file_progress = 100.0 / len(input_files)

        with PdfWriter() as writer:
            for idx, input_file in enumerate(input_files, start=1):
                input_file = Path(input_file)
                self.check_file_exists(input_file)

                with PdfReader(input_file) as reader:
                    if password and reader.is_encrypted:
                        reader.decrypt(password)

                    pages_len = len(reader.pages)
                    for page in reader.pages:
                        writer.add_page(page)

                        completed += file_progress / pages_len
                        if progress_callback:
                            progress_callback(completed)
            if progress_callback:
                progress_callback(100.0)
            writer.write(output_file)

    def split(self,
              output_file: str | Path,
              input_file: str | Path,
              password: str | None = None,
              progress_callback: Callable[[float], Any] | None = None,
              ):
        """
        Split input files into 1-page PDF output files.

        :param output_file: Output PDF file (will be appended with ``_X.pdf`` where ``X`` is the page number)
        :param input_file: Input PDF file
        :param password: Decryption password. Defaults to None.
        :param progress_callback: Progress callback (0-100). Defaults to None.

        :raises FileNotFoundError: if input file not found
        """
        self.check_file_exists(input_file)
        output_path = Path(output_file).with_suffix("")

        with PdfReader(input_file) as reader:
            if password and reader.is_encrypted:
                reader.decrypt(password)

            pages_len = len(reader.pages)
            for i, page in enumerate(reader.pages):
                with PdfWriter() as writer:
                    writer.add_page(page)
                    writer.write(f"{output_path}_{i + 1}.pdf")

                    progress = 100.0 * (float(i) / pages_len)
                    if progress_callback:
                        progress_callback(progress)

    def extract(self,
                output_file: str | Path,
                input_file: str | Path,
                pages: Sequence[int],
                password: str | None = None,
                progress_callback: Callable[[float], Any] | None = None,
                ):
        """
        Extract specific pages from input files into a PDF output file.

        :param output_file: Output PDF file
        :param input_file: Input PDF file
        :param pages: List of pages to extract (0-indexed).
        :param password: Password used to decrypt file, if needed. Defaults to None (do not decrypt).
        :param progress_callback: Progress callback (0-100). Defaults to None.

        :raises FileNotFoundError: if input file not found
        """
        self.check_file_exists(input_file)
        output_path = Path(output_file).with_suffix(".pdf")

        with PdfReader(input_file) as reader, PdfWriter() as writer:
            if password and reader.is_encrypted:
                reader.decrypt(password)

            pdf_len = len(reader.pages)
            pages_len = len(pages)
            for idx, page_num in enumerate(pages, start=1):
                if pdf_len <= page_num:
                    raise ValueError(f"PDF '{input_file}' has only {len(reader.pages)} pages. Cannot extract page {page_num + 1}.")
                writer.add_page(reader.pages[page_num])

                progress = 100.0 * (float(idx) / pages_len)
                if progress_callback:
                    progress_callback(progress)
            if progress_callback:
                progress_callback(100.0)
            writer.write(output_path)

    def rotate(self,
               output_file: str | Path,
               input_file: str | Path,
               rotations: dict[int, int],
               decrypt_password: str | None = None,
               progress_callback: Callable[[float], Any] | None = None,
               ):
        """
        Rotate specific pages from input files, generating a PDF output file.

        :param output_file: Output PDF file.
        :param input_file: Input PDF file.
        :param rotations: Dict format { page_num: rotation_degrees }. Degree must be 0 or multiples of 90 degrees - positive or negative.
        :param decrypt_password: Password used to decrypt file, if needed. Defaults to None (do not decrypt).
        :param progress_callback: Progress callback (0-100). Defaults to None.

        :raises FileNotFoundError: if input file not found
        :raises ValueError: if rotation degree is invalid (valid values are 0 or multiples of 90 degrees - positive or negative).        
        """
        self.check_file_exists(input_file)
        output_path = Path(output_file).with_suffix(".pdf")

        with PdfReader(input_file) as reader, PdfWriter() as writer:
            if decrypt_password and reader.is_encrypted:
                reader.decrypt(decrypt_password)

            pages_len = len(reader.pages)
            for i, page in enumerate(reader.pages):
                rotation = int(rotations.get(i, 0))

                # parse rotation argument
                rotation = normalize_degree(rotation)

                if rotation not in (0, 90, 180, 270):
                    raise ValueError(f"Page '{i}' rotation {rotation} is invalid. Rotation must be 0, 90, 180 or 270 degrees.")

                # execute page rotation
                logger.debug(f"Rotating page {i} by {rotation} deg")
                if rotation > 0:
                    page.rotate(rotation)  # clockwise: 90, 180, 270
                writer.add_page(page)

                progress = 100.0 * (float(i) / pages_len)
                if progress_callback:
                    progress_callback(progress)

            if progress_callback:
                progress_callback(100.0)
            # save output file
            writer.write(output_path)

    def encrypt(self,
                output_file: str | Path,
                input_file: str | Path,
                owner_password: str,
                user_password: str | None = None,
                decrypt_password: str | None = None,
                permission_annotate: bool = False,
                permission_fill_forms: bool = False,
                permission_modify: bool = False,
                permission_modify_pages: bool = False,
                permission_copy: bool = False,
                permission_accessibility: bool = True,
                permission_print_low_quality: bool = True,
                permission_print_high_quality: bool = True,
                permission_all: bool = False,
                encryption_algorithm: EncryptionAlgorithm = EncryptionAlgorithm.AES_256,
                progress_callback: Callable[[float], Any] | None = None,
                ):
        """
        Encrypt input file, generating a protected PDF output file.

        **Available permissions:**
        - ``permission_annotate``: User can add/modify annotations (comments, highlight text, etc) and interactive forms. Default to False (not set).
        - ``permission_fill_forms``: User can fill form fields (subset permission of `permission_annotate`). Default to False (not set).
        - ``permission_modify``: User can modify the document (e.g., add / edit text, add / edit images, etc). Default to False (not set).
        - ``permission_modify_pages``: User can insert, delete, or rotate pages (subset of ``permission_modify``). Default to False (not set).
        - ``permission_copy``: User can copy text/images. Default to False (not set).
        - ``permission_accessibility``: User can use screen readers for accessibility. Default to True (not set).
        - ``permission_print_low_quality``: User can PRINT pdf (low quality). Defaults to True (allow).
        - ``permission_print_high_quality``: User can PRINT pdf (high quality). Requires `permission_print_low_quality=True`. Defaults to True (allow).
        - ``permission_all``: User has ALL PERMISSIONS. If True, it overrides all other permissions. Defaults to False (not set).

        **Encryption algorithms:**
        - ``AES-256``    (high security, compatible with 2008+ PDF readers - **RECOMMENDED**)
        - ``AES-128``    (good security, good compatibility - **RECOMMENDED**)
        - ``AES-256-R5`` (high security,  lowest compatibility - not recommended)
        - ``RC4-128``    (weak security, highest compatibility - not recommended)
        - ``RC4-40``     (weakest security - not recommended)

        :param output_file: Output PDF file.
        :param input_file: Input PDF file.

        :param owner_password: Owner password for encryption. Owner has ALL PERMISSIONS in the output PDF file.
        :param user_password:  User password for encryption. User has ONLY THE PERMISSIONS specified in the arguments. Defaults to None (user and owner password are the same).
        :param decrypt_password: Password used to decrypt file, if needed. Defaults to None (do not decrypt).

        :param permission_annotate: User can add/modify annotations (comments, highlight text, etc) and interactive forms. Default to False (not set).
        :param permission_fill_forms: User can fill form fields (subset permission of `permission_annotate`). Default to False (not set).
        :param permission_modify: User can modify the document (e.g., add / edit text, add / edit images, etc). Default to False (not set).
        :param permission_modify_pages: User can insert, delete, or rotate pages (subset of ``permission_modify``). Default to False (not set).
        :param permission_copy: User can copy text/images. Default to False (not set).
        :param permission_accessibility: User can use screen readers for accessibility. Default to True (allow).
        :param permission_print_low_quality: User can PRINT pdf (low quality). Defaults to True (allow).
        :param permission_print_high_quality: User can PRINT pdf (high quality). Requires `permission_print_low_quality=True`. Defaults to True (allow).
        :param permission_all: User has ALL PERMISSIONS. If True, it overrides all other permissions. Defaults to False (not set).

        :param encryption_algorithm: Encryption algorithm used. Defaults to "AES-256" (for enhanced security and compatibility).
        :param progress_callback: Progress callback (0-100). Defaults to None.

        :raises FileNotFoundError: if input file not found
        """
        self.check_file_exists(input_file)
        output_path = Path(output_file).with_suffix(".pdf")

        with PdfReader(input_file) as reader, PdfWriter() as writer:
            if decrypt_password and reader.is_encrypted:
                reader.decrypt(decrypt_password)

            # set user permissions
            permissions = UserAccessPermissions(0)
            if permission_annotate:
                permissions |= UserAccessPermissions.ADD_OR_MODIFY
            if permission_fill_forms:
                permissions |= UserAccessPermissions.FILL_FORM_FIELDS
            if permission_modify:
                permissions |= UserAccessPermissions.MODIFY
            if permission_modify_pages:
                permissions |= UserAccessPermissions.ASSEMBLE_DOC
            if permission_copy:
                permissions |= UserAccessPermissions.EXTRACT
            if permission_accessibility:
                permissions |= UserAccessPermissions.EXTRACT_TEXT_AND_GRAPHICS
            if permission_print_low_quality:
                permissions |= UserAccessPermissions.PRINT
            if permission_print_high_quality:
                permissions |= UserAccessPermissions.PRINT_TO_REPRESENTATION
            if permission_all:
                permissions |= UserAccessPermissions.all()

            # add pages into writer
            pages_len = len(reader.pages)
            for i, page in enumerate(reader.pages, start=0):
                writer.add_page(page)
                progress = 100.0 * (float(i) / pages_len)
                if progress_callback:
                    progress_callback(progress)

            # Encrypt with user and owner passwords
            writer.encrypt(
                owner_password=owner_password,
                user_password=user_password if user_password else owner_password,
                permissions_flag=permissions,
                algorithm=encryption_algorithm.value,
                use_128bit=True,
            )

            # save output file
            writer.write(output_path)

            if progress_callback:
                progress_callback(100.0)

    def decrypt(self,
                output_file: str | Path,
                input_file: str | Path,
                password: str,
                progress_callback: Callable[[float], Any] | None = None,
                ):
        """
        Decrypt input file, generating a non-protected PDF output file.

        :param output_file: Output PDF file.
        :param input_file: Input PDF file.
        :param password: Password used to decrypt file.
        :param progress_callback: Progress callback (0-100). Defaults to None.

        :raises FileNotFoundError: if input file not found
        """
        self.check_file_exists(input_file)
        output_path = Path(output_file).with_suffix(".pdf")

        with PdfReader(input_file) as reader, PdfWriter() as writer:
            # decrypt file
            if reader.is_encrypted:
                reader.decrypt(password)

            # Generate decrypted pdf
            pages_len = len(reader.pages)
            for i, page in enumerate(reader.pages, start=0):
                writer.add_page(page)
                progress = 100.0 * (float(i) / pages_len)
                if progress_callback:
                    progress_callback(progress)

            # save file
            writer.write(output_path)
            if progress_callback:
                progress_callback(100.0)



# src/file_conversor/backend/gui/pdf/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui.pdf._index import pdf_index

from file_conversor.backend.gui.pdf.compress import pdf_compress
from file_conversor.backend.gui.pdf.convert import pdf_convert
from file_conversor.backend.gui.pdf.decrypt import pdf_decrypt
from file_conversor.backend.gui.pdf.encrypt import pdf_encrypt
from file_conversor.backend.gui.pdf.extract_img import pdf_extract_img
from file_conversor.backend.gui.pdf.extract import pdf_extract
from file_conversor.backend.gui.pdf.merge import pdf_merge
from file_conversor.backend.gui.pdf.ocr import pdf_ocr
from file_conversor.backend.gui.pdf.repair import pdf_repair
from file_conversor.backend.gui.pdf.rotate import pdf_rotate
from file_conversor.backend.gui.pdf.split import pdf_split


def routes():
    return [
        FlaskRoute(
            rule="/pdf",
            handler=pdf_index,
        ),

        # TOOLS
        FlaskRoute(
            rule="/pdf/compress",
            handler=pdf_compress,
        ),
        FlaskRoute(
            rule="/pdf/convert",
            handler=pdf_convert,
        ),
        FlaskRoute(
            rule="/pdf/decrypt",
            handler=pdf_decrypt,
        ),
        FlaskRoute(
            rule="/pdf/encrypt",
            handler=pdf_encrypt,
        ),
        FlaskRoute(
            rule="/pdf/extract_img",
            handler=pdf_extract_img,
        ),
        FlaskRoute(
            rule="/pdf/extract",
            handler=pdf_extract,
        ),
        FlaskRoute(
            rule="/pdf/merge",
            handler=pdf_merge,
        ),
        FlaskRoute(
            rule="/pdf/ocr",
            handler=pdf_ocr,
        ),
        FlaskRoute(
            rule="/pdf/repair",
            handler=pdf_repair,
        ),
        FlaskRoute(
            rule="/pdf/rotate",
            handler=pdf_rotate,
        ),
        FlaskRoute(
            rule="/pdf/split",
            handler=pdf_split,
        ),
    ]


__all__ = ['routes']




# src\file_conversor\cli\gui\start_cmd.py

import typer

from typing import Annotated

from rich import print


# user-provided modules
from file_conversor.backend.gui import WebApp

from file_conversor.cli.gui._typer import COMMAND_NAME, START_NAME

from file_conversor.config import Configuration, State, Log
from file_conversor.config.locale import get_translation

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = set([])


@typer_cmd.command(
    name=START_NAME,
    help=f"""
        {_('Starts the graphical user interface.')}
    """,
    epilog=f"""
**{_('Examples')}:** 

- `file_conversor {COMMAND_NAME} {START_NAME}`
    """
)
def start_gui():
    logger.info(f"[bold]{_('Starting the graphical user interface')} ...[/]")
    webapp = WebApp.get_instance()
    webapp.run()



# src\file_conversor\backend\gifsicle_backend.py

"""
This module provides functionalities for handling files using gifsicle.
"""

from pathlib import Path
from rich import print

# user-provided imports
from file_conversor.config import Environment, Log
from file_conversor.config.locale import get_translation

from file_conversor.dependency import BrewPackageManager, ScoopPackageManager
from file_conversor.backend.abstract_backend import AbstractBackend

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class _GifSicleBackend(AbstractBackend):  # pyright: ignore[reportUnusedClass]
    """
    Provides an interface for handling files using gifsicle.
    """

    SUPPORTED_IN_FORMATS = {
        'gif': {},
    }
    SUPPORTED_OUT_FORMATS = {
        'gif': {},
    }
    EXTERNAL_DEPENDENCIES = set([
        "gifsicle",
    ])

    def __init__(
        self,
        install_deps: bool | None,
        verbose: bool = False,
    ):
        """
        Initialize the backend.

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 
        :param verbose: Verbose logging. Defaults to False.      

        :raises RuntimeError: if dependency is not found
        """
        super().__init__(
            pkg_managers={
                ScoopPackageManager({
                    "gifsicle": "gifsicle"
                }),
                BrewPackageManager({
                    "gifsicle": "gifsicle"
                }),
            },
            install_answer=install_deps,
        )
        self._verbose = verbose

        # check binary
        self._bin = self.find_in_path("gifsicle")

    def compress(
        self,
            input_file: str | Path,
            output_file: str | Path,
            compression_level: int = 3,
            **kwargs,
    ):
        """
        Execute command to compress the input file.

        :param input_file: Input file path.
        :param output_file: Output file path.              
        :param compression_level: Image compression level (0-3). Defaults to 3 (max compression).              
        :param kwargs: Optional arguments.

        :return: Subprocess.CompletedProcess object
        """

        # Execute the command
        process = Environment.run(
            f"{self._bin}",
            f"-O={compression_level}",
            f"{input_file}",
            f"-o", f"{output_file}",
        )
        return process



# src/file_conversor/backend/gui/_api/pdf/encrypt.py

from flask import json, render_template, request, url_for
from pathlib import Path
from typing import Any

# user-provided modules
from file_conversor.backend.gui.flask_api import FlaskApi
from file_conversor.backend.gui.flask_api_status import FlaskApiStatus

from file_conversor.cli.pdf.encrypt_cmd import execute_pdf_encrypt_cmd

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def _api_thread(params: dict[str, Any], status: FlaskApiStatus) -> None:
    """Thread to handle PDF encryption."""
    logger.debug(f"PDF encryption thread received: {params}")
    input_files: list[Path] = [Path(i) for i in params['input-files']]
    output_dir: Path = Path(params['output-dir'])

    encrypt_algo: str = str(params['pdf-encryption-algorithm'])
    decrypt_password: str | None = params.get('decrypt-password')
    owner_password: str = str(params['owner-password'])
    user_password: str | None = str(params['user-password']) or None

    allow_annotations: bool = bool(params['allow-annotations'])
    allow_fill_forms: bool = bool(params['allow-fill-forms'])
    allow_modify: bool = bool(params['allow-modify'])
    allow_modify_pages: bool = bool(params['allow-modify-pages'])
    allow_copy: bool = bool(params['allow-copy'])
    allow_accessibility: bool = bool(params['allow-accessibility'])
    allow_print_lq: bool = bool(params['allow-print-lq'])
    allow_print_hq: bool = bool(params['allow-print-hq'])

    execute_pdf_encrypt_cmd(
        input_files=input_files,
        decrypt_password=decrypt_password,
        owner_password=owner_password,
        user_password=user_password,
        allow_annotate=allow_annotations,
        allow_fill_forms=allow_fill_forms,
        allow_modify=allow_modify,
        allow_modify_pages=allow_modify_pages,
        allow_copy=allow_copy,
        allow_accessibility=allow_accessibility,
        allow_print_lq=allow_print_lq,
        allow_print_hq=allow_print_hq,
        allow_all=False,
        encrypt_algo=encrypt_algo,
        output_dir=output_dir,
        progress_callback=status.set_progress,
    )


def api_pdf_encrypt():
    """API endpoint to encrypt PDF documents."""
    logger.info(f"[bold]{_('PDF encryption requested via API.')}[/]")
    return FlaskApi.execute_response(_api_thread)




# src\file_conversor\cli\config\__init__.py


import typer

# user-provided modules
from file_conversor.config.locale import get_translation

from file_conversor.cli.config._typer import COMMAND_NAME

from file_conversor.cli.config.set_cmd import typer_cmd as set_cmd
from file_conversor.cli.config.show_cmd import typer_cmd as show_cmd

_ = get_translation()

config_cmd = typer.Typer(
    name=COMMAND_NAME,
    help=_("Configure default options"),
)
config_cmd.add_typer(show_cmd)
config_cmd.add_typer(set_cmd)



# src\file_conversor\utils\bulma\form_checkbox.py

from typing import Any

from file_conversor.utils.dominate_bulma.font_awesome_icon import FontAwesomeIcon

from file_conversor.utils.dominate_bulma.form_field import FormField

from file_conversor.utils.dominate_utils import *


def Checkbox(
    label_text: str,
    _class_container: str = "",
    _class_span: str = "",
    reverse: bool = False,
    **kwargs,
):
    """
    Create a checkbox input.

    :param label_text: The text for the checkbox label.
    :param _class_container: Additional CSS classes for the label container.
    :param _class_span: Additional CSS classes for the span containing the label text.
    :param reverse: Whether to reverse the order of the checkbox and label.    
    """
    with label(_class=f"checkbox {_class_container}") as checkbox_label:
        if reverse:
            span(label_text, _class=_class_span)
        input_(_type=f"checkbox", **kwargs)
        if not reverse:
            span(label_text, _class=_class_span)
    return checkbox_label


def FormFieldCheckbox(
    _name: str,
    current_value: str,
    help: str,
    label_text: str = "",
    reverse: bool = False,
    x_data: str = "",
    x_init: str = "",
    **kwargs,
):
    """
    Create a form field with a label and a select element.

    :param _name: The name attribute for the select element.
    :param current_value: The current selected value.
    :param help: Optional help text.
    :param label_text: The text for the label.
    :param reverse: Whether to reverse the order of the checkbox and label.
    :param x_data: Additional Alpine.js x-data properties.
    :param x_init: Additional Alpine.js x-init script.

    :return: The FormField element.
    """
    field = FormField(
        Checkbox(
            label_text=label_text,
            _class_container=f"""
                is-flex 
                is-justify-content-flex-end 
                is-align-items-center
            """,
            _class=f"{'ml-1' if reverse else 'mr-1'}",
            _style="margin-top: 3px;",
            _name=_name,
            _title=help,
            reverse=reverse,
            **{
                ':class': """{
                    'is-danger': !isValid,
                    'is-success': isValid,
                }""",
                'x-model': 'value',
            },
            **kwargs,
        ),
        _class_control="is-flex is-flex-direction-column is-flex-grow-1",
        current_value=True if current_value in ["true", "on", "yes", True] else False,
        help=help,
        x_data=x_data,
        x_init=x_init,
    )
    return field


__all__ = [
    "Checkbox",
    "FormFieldCheckbox",
]



# src/file_conversor/backend/gui/_api/video/info.py

from flask import json, render_template, request, url_for
from pathlib import Path
from typing import Any

# user-provided modules
from file_conversor.backend.gui.flask_api import FlaskApi
from file_conversor.backend.gui.flask_api_status import FlaskApiStatus

from file_conversor.backend.audio_video import FFprobeBackend

from file_conversor.utils.backend import FFprobeParser
from file_conversor.utils.bulma_utils import *
from file_conversor.utils.dominate_utils import *

from file_conversor.utils.formatters import format_bitrate, format_bytes

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def _api_thread(params: dict[str, Any], status: FlaskApiStatus) -> None:
    """Thread to handle audio information retrieval."""
    logger.debug(f"Audio info thread received: {params}")
    input_files: list[Path] = [Path(i) for i in params['input-files']]

    backend = FFprobeBackend(
        install_deps=CONFIG['install-deps'],
        verbose=STATE["verbose"],
    )

    with div() as result:
        for input_file in input_files:
            logger.info(f"[bold]{_('Retrieving info for')}[/] [green]{input_file.name}[/]...")
            parser = FFprobeParser(backend, input_file)
            parser.run()

            parser.get_chapters().div()
            parser.get_format().div()
            parser.get_streams().div()
            div("----------------------------------------")
            br()

    status.set_message(str(result))
    status.set_progress(100)
    logger.debug(f"{status}")


def api_video_info():
    """API endpoint to get video file information."""
    logger.info(f"[bold]{_('Video info requested via API.')}[/]")
    return FlaskApi.execute_response(_api_thread)



# src\file_conversor\system\__init__.py

"""Stores platform specific methods"""

import platform
from rich import print

_PLATFORM_WINDOWS = "Windows"
_PLATFORM_LINUX = "Linux"
_PLATFORM_MACOS = "Darwin"
_PLATFORM_UNKNOWN = ""

_CURR_PLATFORM = platform.system()


def is_windows():
    return _CURR_PLATFORM == _PLATFORM_WINDOWS


def is_linux():
    return _CURR_PLATFORM == _PLATFORM_LINUX


def is_macos():
    return _CURR_PLATFORM == _PLATFORM_MACOS


def is_unknown():
    return _CURR_PLATFORM == _PLATFORM_UNKNOWN


# dynamically load modules, as needed
if _CURR_PLATFORM == _PLATFORM_WINDOWS:
    # WINDOWS OS
    from file_conversor.system.win import *
    print("[bold blue]System detected[/]: Windows.")

elif _CURR_PLATFORM == _PLATFORM_LINUX:
    # LINUX OS
    from file_conversor.system.lin import *
    print("[bold blue]System detected[/]: Linux.")

elif _CURR_PLATFORM == _PLATFORM_MACOS:
    # MACOS OS
    from file_conversor.system.mac import *
    print("[bold blue]System detected[/]: macOS.")

else:
    # UNKNOWN OS
    _CURR_PLATFORM = _PLATFORM_UNKNOWN
    from file_conversor.system.dummy import *
    print("[bold blue]System detected[/]: Unknown OS.")



# src\file_conversor\cli\hash\__init__.py

import typer

# user-provided modules
from file_conversor.config.locale import get_translation

from file_conversor.cli.hash._typer import COMMAND_NAME

from file_conversor.cli.hash.check_cmd import typer_cmd as check_cmd
from file_conversor.cli.hash.create_cmd import typer_cmd as create_cmd

_ = get_translation()

hash_cmd = typer.Typer(
    name=COMMAND_NAME,
    help=_("Hashing manipulation (check, gen, etc)"),
)
hash_cmd.add_typer(create_cmd)
hash_cmd.add_typer(check_cmd)



# src\file_conversor\utils\bulma_utils\general.py

from pathlib import Path
from typing import Any, Sequence

# user-provided modules
from file_conversor.utils.dominate_bulma.form_field import FormFieldHorizontal
from file_conversor.utils.dominate_bulma.form_select import FormFieldSelect
from file_conversor.utils.dominate_bulma.form_checkbox import FormFieldCheckbox
from file_conversor.utils.dominate_bulma.form_file_list import FormFileList
from file_conversor.utils.dominate_bulma.form_output_dir import FormFieldOutputDirectory
from file_conversor.utils.dominate_bulma.form_output_file import FormFieldOutputFile

from file_conversor.utils.dominate_utils import *

from file_conversor.utils.formatters import format_file_types_webview

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


def InputFilesField(
        *file_types: str,
        description: str = "",
):
    """
    Create a FormFieldInput for file list input.

    :param file_types: File type patterns (e.g., '.png', '.jpg').
    :param description: Description for the file types (e.g., 'Image Files').
    """
    return FormFileList(
        input_name="input-files",
        validation_expr="value.length > 0",
        label_text=_("Input Files"),
        help_text=_("Select (or drag) the input files."),
        btn_add_help=_("Add file"),
        btn_remove_help=_("Remove file"),
        file_types=[
            format_file_types_webview(
                *file_types,
                description=description,
            ),
        ],
        multiple=True,
    )


def OutputDirField(
    _name: str = "output-dir",
    help: str = _("Select the output directory to save the file."),
    label_text: str = _("Output Directory"),
):
    return FormFieldHorizontal(
        FormFieldOutputDirectory(
            _name=_name,
            help=help,
        ),
        label_text=label_text,
    )


def OutputFileField(
    *file_types: tuple[str, str],
    current_value: str = "",
):
    """
    Create a FormFieldOutputFile for selecting output file.

    :param file_types: A list of tuples with format (file_type, description).
    :param current_value: The current output filename. If provided, it will be appended to the last opened directory.
    """
    return FormFieldHorizontal(
        FormFieldOutputFile(
            _name="output-file",
            help=_("Select the output filename to save the file."),
            file_types=[
                format_file_types_webview(
                    ftype,
                    description=desc,
                ) for ftype, desc in file_types
            ],
            current_value=current_value,
        ),
        label_text=_("Output File"),
    )


def FileFormatField(
    *options: tuple[str, str],
    current_value: str | None = None,
):
    """
    Create a form field for file format selection.

    :param options: A list of tuples where each tuple contains (value, display_text)
    :param current_value: The currently selected value.
    """
    return FormFieldHorizontal(
        FormFieldSelect(
            *options,
            current_value=current_value,
            _name="file-format",
            help=_("Select the output file format."),
        ),
        label_text=_("Output Format"),
    )


def OverwriteFilesField():
    """Create a form field for overwrite option."""
    return FormFieldCheckbox(
        current_value=STATE["overwrite-output"],
        _name="overwrite-output",
        label_text=_("Overwrite Existing Files"),
        help=_("Allow overwriting of existing files."),
    )


__all__ = [
    "InputFilesField",
    "OutputDirField",
    "OutputFileField",
    "FileFormatField",
    "OverwriteFilesField",
]




# src\file_conversor\cli\pdf\ocr_cmd.py

import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List

from rich import print

# user-provided modules
from file_conversor.backend.pdf import OcrMyPDFBackend

from file_conversor.cli.pdf._typer import TRANSFORMATION_PANEL as RICH_HELP_PANEL
from file_conversor.cli.pdf._typer import COMMAND_NAME, OCR_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.rich_utils import get_progress_bar
from file_conversor.utils.typer_utils import DPIOption, FormatOption, InputFilesArgument, OutputDirOption

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = OcrMyPDFBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    for ext in OcrMyPDFBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="ocr",
                description="OCR",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{OCR_NAME}" "%1"',
                icon=str(icons_folder_path / 'ocr.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_pdf_ocr_cmd(
    input_files: List[Path],
    languages: List[str],
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    ocrmypdf_backend = OcrMyPDFBackend(
        install_deps=CONFIG['install-deps'],
        verbose=STATE['verbose'],
    )
    local_langs: set[str] = ocrmypdf_backend.get_available_languages()
    remote_langs: set[str] = set()

    if 'all' in languages:
        remote_langs = ocrmypdf_backend.get_available_remote_languages()
        print(f"{_('Available remote languages')}: {', '.join(remote_langs)}")
        print(f"{_('Installed languages')}: {', '.join(local_langs)}")
        return

    install_langs = set(languages) - local_langs
    if install_langs:
        remote_langs = ocrmypdf_backend.get_available_remote_languages()
        if install_langs - remote_langs:
            print(f"{_('Available remote languages')}: {', '.join(remote_langs)}")
            print(f"{_('Languages requested')}: {', '.join(install_langs)}")
            raise ValueError(f"{_('Some languages are not available for installation')}.")

        with get_progress_bar() as progress:
            for lang in install_langs:
                task = progress.add_task(f"{_('Installing language')} '{lang}' ...", total=100)
                ocrmypdf_backend.install_language(
                    lang=lang,
                    progress_callback=lambda p: progress.update(task, completed=p),
                )
                progress.update(task, completed=100)

    for idx, input_file in enumerate(input_files):
        input_file = Path(input_file).resolve()
        output_file = output_dir / CommandManager.get_output_file(input_file, stem="_ocr")
        if not STATE["overwrite-output"] and output_file.exists():
            raise FileExistsError(f"{_("File")} '{output_file}' {_("exists")}. {_("Use")} 'file_conversor -oo' {_("to overwrite")}.")

        print(f"Processing '{output_file}' ...")

        ocrmypdf_backend.to_pdf(
            input_file=input_file,
            output_file=output_file,
            languages=languages,
        )
        progress_callback(100.0 * (float(idx + 1) / len(input_files)))

    logger.info(f"{_('File OCR')}: [green][bold]{_('SUCCESS')}[/bold][/green]")


@typer_cmd.command(
    name=OCR_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Create a searchable PDF file from scanned documents using OCR.')}

        {_('Outputs a text searchable PDF file.')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {OCR_NAME} input_file.pdf -l all`

        - `file_conversor {COMMAND_NAME} {OCR_NAME} input_file.pdf -l eng`

        - `file_conversor {COMMAND_NAME} {OCR_NAME} input_file.pdf input_file2.pdf -l eng -l por`

        - `file_conversor {COMMAND_NAME} {OCR_NAME} input_file.pdf input_file2.pdf -l eng -od "D:\\Downloads"`
    """)
def ocr(
    input_files: Annotated[List[Path], InputFilesArgument(OcrMyPDFBackend)],

    languages: Annotated[List[str], typer.Option(
        "--languages", "-l",
        help=_("Languages to use for OCR (three character language codes). Format: LANG (e.g., 'eng', 'por'). Type 'all' to query all available languages."),
    )],

    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_pdf_ocr_cmd(
        input_files=input_files,
        languages=languages,
        output_dir=output_dir,
    )




# src\file_conversor\cli\multimedia\antialias_cmd.py
import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List

from rich import print

# user-provided modules
from file_conversor.backend.image import PillowBackend
from file_conversor.cli.image._typer import FILTER_PANEL as RICH_HELP_PANEL
from file_conversor.cli.image._typer import COMMAND_NAME, ANTIALIAS_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import InputFilesArgument, OutputDirOption, RadiusOption
from file_conversor.utils.validators import check_is_bool_or_none, check_path_exists, check_valid_options

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = PillowBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    # IMG2PDF commands
    for ext in PillowBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="antialias",
                description="Antialias",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{ANTIALIAS_NAME}" "%1" -r 3',
                icon=str(icons_folder_path / "diagonal_line.ico"),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_image_antialias_cmd(
    input_files: List[Path],
    radius: int,
    algorithm: str,
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    pillow_backend = PillowBackend(verbose=STATE['verbose'])

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        logger.info(f"Processing '{output_file}' ... ")
        pillow_backend.antialias(
            input_file=input_file,
            output_file=output_file,
            radius=radius,
            algorithm=PillowBackend.AntialiasAlgorithm.from_str(algorithm),
        )
        progress_callback(progress_mgr.complete_step())

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_stem="_antialiased")

    logger.info(f"{_('Image antialiasing')}: [green bold]{_('SUCCESS')}[/]")


@typer_cmd.command(
    name=ANTIALIAS_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Applies antialias filter to an image file.')}
    """,
    epilog=f"""
        **{_('Examples')}:**

        - `file_conversor {COMMAND_NAME} {ANTIALIAS_NAME} input_file1.bmp -r 5`

        - `file_conversor {COMMAND_NAME} {ANTIALIAS_NAME} input_file.jpg -r 2 -a mode`        
    """)
def antialias(
    input_files: Annotated[List[Path], InputFilesArgument(PillowBackend)],

    radius: Annotated[int, RadiusOption()] = 3,

    algorithm: Annotated[str, typer.Option("--algorithm", "-a",
                                           help=f'{_("Algorithm to use. Available algorihtms:")} {", ".join(PillowBackend.AntialiasAlgorithm.get_dict())}.',
                                           callback=lambda x: check_valid_options(x, PillowBackend.AntialiasAlgorithm.get_dict()),
                                           )] = "median",

    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_image_antialias_cmd(
        input_files=input_files,
        radius=radius,
        algorithm=algorithm,
        output_dir=output_dir,
    )



# src/file_conversor/backend/gui/hash/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui.hash._index import hash_index

from file_conversor.backend.gui.hash.check import hash_check
from file_conversor.backend.gui.hash.create import hash_create


def routes():
    return [
        FlaskRoute(
            rule="/hash",
            handler=hash_index
        ),
        FlaskRoute(
            rule="/hash/check",
            handler=hash_check
        ),
        FlaskRoute(
            rule="/hash/create",
            handler=hash_create
        ),
    ]


__all__ = ['routes']



# src\file_conversor\backend\gui\flask_route.py

from flask import Flask
from typing import Any, Callable


class FlaskRoute:
    def __init__(self, rule: str, handler: Callable[..., Any], **options: Any) -> None:
        super().__init__()
        self.rule = rule
        self.handler = handler
        self.options = options

    def __hash__(self) -> int:
        return hash(self.rule)

    def __eq__(self, value: object) -> bool:
        if not isinstance(value, FlaskRoute):
            return NotImplemented
        return self.rule == value.rule

    def __repr__(self) -> str:
        return f"FlaskRoute(rule={self.rule}, options={self.options})"

    def __str__(self) -> str:
        return f"{self.rule}" + f"({self.options})" if self.options else ""

    def register(self, app: Flask) -> None:
        app.route(self.rule, **self.options)(self.handler)


__all__ = ["FlaskRoute"]




# src\file_conversor\cli\image\info_cmd.py

import typer

from pathlib import Path
from typing import Annotated, List

from rich import print
from rich.panel import Panel
from rich.console import Group

# user-provided modules
from file_conversor.backend.image import PillowBackend

from file_conversor.cli.image._typer import OTHERS_PANEL as RICH_HELP_PANEL
from file_conversor.cli.image._typer import COMMAND_NAME, INFO_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils.backend import PillowParser
from file_conversor.utils.typer_utils import InputFilesArgument

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = PillowBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    # Pillow commands
    for ext in PillowBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="info",
                description="Get Info",
                command=f'cmd /k "{Environment.get_executable()} "{COMMAND_NAME}" "{INFO_NAME}" "%1""',
                icon=str(icons_folder_path / "info.ico"),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


@typer_cmd.command(
    name=INFO_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Get EXIF information about a image file.')}

        {_('This command retrieves metadata and other information about the image file')}:
    """,
    epilog=f"""
        **{_('Examples')}:**

        - `file_conversor {COMMAND_NAME} {INFO_NAME} other_filename.jpg`

        - `file_conversor {COMMAND_NAME} {INFO_NAME} filename.webp filename2.png filename3.gif`
    """)
def info(
    input_files: Annotated[List[str], InputFilesArgument(PillowBackend)],
):
    pillow_backend = PillowBackend(verbose=STATE['verbose'])
    for input_file in input_files:

        # 📁 Informações gerais do arquivo
        parser = PillowParser(pillow_backend, Path(input_file))
        parser.run()

        # Agrupar e exibir tudo com Rich
        group = Group(
            *parser.get_exif_info().rich(),
        )
        print(Panel(group, title=f"🧾 {_('File Analysis')}", border_style="blue"))



# src\file_conversor\backend\ocrmypdf_backend.py

import multiprocessing
import re
import ocrmypdf

from pathlib import Path

from typing import Any, Callable

# user-provided imports
from file_conversor.config import Environment, Log, State
from file_conversor.config.locale import get_translation

from file_conversor.backend.abstract_backend import AbstractBackend
from file_conversor.backend.git_backend import GitBackend
from file_conversor.backend.http_backend import HttpBackend

from file_conversor.dependency import BrewPackageManager, ScoopPackageManager

STATE = State.get_instance()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)
_ = get_translation()


class OcrMyPDFBackend(AbstractBackend):
    TESSDATA_REMOTE_LANGS: set[str] | None = None
    TESSDATA_DIR: Path | None = None

    TESSDATA_REPOSITORY = {
        "user_name": "tesseract-ocr",
        "repo_name": "tessdata",
        "branch": "main",
    }

    SUPPORTED_IN_FORMATS = {
        "pdf": {},
    }
    SUPPORTED_OUT_FORMATS = {
        "pdf": {},
    }
    EXTERNAL_DEPENDENCIES = {
        "tesseract",
    }

    def __init__(
        self,
        install_deps: bool | None,
        verbose: bool = False,
    ):
        """
        Initialize the backend.

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 

        :raises RuntimeError: if dependency is not found
        """
        super().__init__(
            pkg_managers={
                ScoopPackageManager({
                    "tesseract": "tesseract",
                }),
                BrewPackageManager({
                    "tesseract": "tesseract"
                }),
            },
            install_answer=install_deps,
        )
        self._install_deps = install_deps
        self._verbose = verbose

        self._tesseract_bin = self.find_in_path("tesseract")

        self.TESSDATA_DIR = self.get_tessdata_dir()
        logger.debug(f"{_('Tesseract tessdata directory')}: {self.TESSDATA_DIR}")

    def install_language(
            self,
            lang: str,
            progress_callback: Callable[[float], None] | None = None,
    ):
        if not self.TESSDATA_DIR:
            raise FileNotFoundError(_("Tessdata directory not found."))

        lang_file = self.TESSDATA_DIR / f"{lang}.traineddata"
        if lang_file.exists():
            logger.warning(f"{_('Language')} '{lang}' {_('already installed')}.")
            return

        lang_url = GitBackend.get_download_url(
            **self.TESSDATA_REPOSITORY,
            file_path=f"{lang}.traineddata",
        )
        http_backend = HttpBackend(verbose=self._verbose)
        http_backend.download(
            url=lang_url,
            dest_folder=lang_file,
            progress_callback=progress_callback,
        )

        available_languages = self.get_available_languages()
        if lang not in available_languages:
            raise RuntimeError(f"{_('Failed to install language')} '{lang}'.")

    def get_tessdata_dir(self) -> Path:
        """
        Get the tessdata directory.

        :return: Path to tessdata directory.

        :raises FileNotFoundError: if tessdata directory not found
        """
        if self.TESSDATA_DIR:
            return self.TESSDATA_DIR

        process = Environment.run(
            str(self._tesseract_bin),
            "--list-langs",
        )
        lines: list[str] = process.stdout.splitlines()
        for line in lines:
            match = re.match(r"^List of available languages in \"(.+)\"", line)
            if not match:
                continue
            tessdata_path = Path(match.group(1).strip()).resolve()
            if not tessdata_path.exists():
                raise FileNotFoundError(f"Tessdata path '{tessdata_path}' does not exist.")
            return tessdata_path
        raise FileNotFoundError(_("Tessdata directory not found."))

    def get_available_remote_languages(self) -> set[str]:
        """
        Get available remote languages for OCR.
        """
        if self.TESSDATA_REMOTE_LANGS is not None:
            return self.TESSDATA_REMOTE_LANGS

        self.TESSDATA_REMOTE_LANGS = set()
        for file_info in GitBackend.get_info_api(
            **self.TESSDATA_REPOSITORY,
        ):
            if not file_info.get("name", "").endswith(".traineddata"):
                continue
            lang = file_info["name"][:-len(".traineddata")]
            if lang and lang not in ("configs", "tessdata_best", "tessdata_fast"):
                self.TESSDATA_REMOTE_LANGS.add(lang)
        return self.TESSDATA_REMOTE_LANGS

    def get_available_languages(self) -> set[str]:
        """
        Get available languages for OCR.

        :return: List of available languages.
        """
        process = Environment.run(
            str(self._tesseract_bin),
            "--list-langs",
        )
        # First line is usually 'List of available languages (x):'
        langs: set[str] = set()
        for line in process.stdout.splitlines()[1:]:
            line = str(line).strip().lower()
            if not line or line == "none" or line.startswith("list of available"):
                continue
            langs.add(line)
        return langs

    def to_pdf(
        self,
        output_file: str | Path,
        input_file: str | Path,
        languages: list[str],
        num_processses: int = multiprocessing.cpu_count(),
    ):
        """
        OCR input files into output file.

        :param output_file: Output file
        :param input_file: Input file. 
        :param languages: Languages to use in OCR
        :param num_processes: Number of processes to use. Defaults to max number of CPU cores.

        :raises FileNotFoundError: if input file not found
        """
        input_file = Path(input_file).resolve()
        output_file = Path(output_file).resolve()

        self.check_file_exists(input_file)

        ocrmypdf.ocr(
            input_file=input_file,
            output_file=output_file,
            language=languages,
            jobs=max(1, num_processses),
            use_threads=False,  # use processes instead of threads due to GIL
            skip_text=True,  # skip OCR if text is already present
            progress_bar=True,
        )



# src\file_conversor\utils\bulma\form.py

from typing import Any

from file_conversor.utils.dominate_bulma.font_awesome_icon import FontAwesomeIcon
from file_conversor.utils.dominate_utils import *

from file_conversor.utils.formatters import format_py_to_js


def FormField(
    *input_el,
    label_text: str = "",
    help: str = "",
    icons: dict[str, Any] | None = None,
    current_value: Any = None,
    validation_expr: str = "true",
    has_addons: bool = False,
    _class: str = "",
    _class_control: str = "",
    x_data: str = "",
    x_init: str = "",
    **kwargs,
):
    """
    Create a form field with a label and optional icons.

    :param input_el: The input elements (e.g., input, textarea, select).
    :param label_text: The text for the label.
    :param help: Optional help text.
    :param icons: Optional icons for left and right (e.g., {"left": left_icon_name, "right": right_icon_name}).
    :param _class: Additional CSS classes for the form field.
    :param _class_control: Additional CSS classes for the control div.
    :param x_data: Additional Alpine.js x-data properties.
    :param x_init: Additional Alpine.js x-init code.
    """
    with div(
        _class=f"field is-full-width {_class}",
        **{
            'x-data': f"""{{
                help: {format_py_to_js(help)},
                value: {format_py_to_js(current_value)},
                isValid: false,
                validate(value){{
                    console.log('Validating field with value:', value);
                    this.isValid = {validation_expr} ;
                    const parentForm = this.$el.closest('form[x-data]');
                    if(parentForm){{
                        const parentData = Alpine.$data(parentForm);
                        parentData.updateValidity();
                    }} else {{
                        console.log('No parent form found');
                    }}
                    return this.isValid ;
                }},
                init() {{
                    this.$watch('value', this.validate.bind(this));
                    this.validate(this.value);   
                    {x_init} ;                 
                }},
                {x_data}
            }}""",
        },
        **kwargs,
    ) as field:
        if label_text:
            label(label_text, _class="label")
        if "is-grouped" in _class:
            for el in input_el:
                if not el:
                    continue
                with div(_class=f"control {_class_control}") as control_group:
                    control_group.add(el)
        elif has_addons:
            with div(_class=f"field has-addons") as control_group:
                for el in input_el:
                    if not el:
                        continue
                    control_group.add(el)
        else:
            with div(_class=f"control {'has-icons-left' if icons and icons.get('left') else ''} {'has-icons-right' if icons and icons.get('right') else ''} {_class_control}") as control:
                for el in input_el:
                    if not el:
                        continue
                    control.add(el)
                if icons and icons.get('left'):
                    FontAwesomeIcon(icons['left'], _class="is-left is-small")
                if icons and icons.get('right'):
                    FontAwesomeIcon(icons['right'], _class="is-right is-small")
        p(
            _class=f"help is-danger",
            **{
                'x-text': 'help',
                ':class': """{
                    'is-hidden': isValid,
                }""",
            },
        )  # Placeholder for error messages
    return field


def FormFieldHorizontal(
    *form_fields,
    label_text: str,
    _class: str = "",
    _class_label: str = "is-normal",
    _class_body: str = "",
    **kwargs,
):
    """
    Create a horizontal form field with a label and multiple input elements.

    :param form_fields: The form field elements (e.g., FormField).
    :param label_text: The text for the label.
    :param _class: Additional CSS classes for the form field.
    :param _class_label: Additional CSS classes for the label container.
    :param _class_body: Additional CSS classes for the body container.
    :param kwargs: Additional attributes for the field container.
    """
    with div(_class=f"field is-horizontal is-full-width {_class}", **kwargs) as field:
        with div(_class=f"field-label {_class_label}") as field_label:
            label(label_text, _class="label")
        with div(_class=f"field-body {_class_body}") as field_body:
            for form_field in form_fields:
                field_body.add(form_field)
    return field


__all__ = [
    "FormFieldHorizontal",
    "FormField",
]



# src\file_conversor\backend\gui\_components\level.py

from typing import Any

from file_conversor.utils.dominate_utils import *


def Level(
        center: list[Any] | None = None,
        left: list[Any] | None = None,
        right: list[Any] | None = None,
        **kwargs
):
    with div(_class="level", **kwargs) as level:
        if center:
            with div(_class="level-item has-text-centered") as level_item:
                for item in center:
                    level_item.add(item)
        if left:
            with div(_class="level-left"):
                for item in left:
                    with div(_class="level-item") as level_item:
                        level_item.add(item)
        if right:
            with div(_class="level-right"):
                for item in right:
                    with div(_class="level-item") as level_item:
                        level_item.add(item)
    return level


__all__ = [
    'Level',
]



# src\file_conversor\cli\pipeline\__init__.py

import typer

# user-provided modules
from file_conversor.config import get_translation

from file_conversor.cli.pipeline._typer import COMMAND_NAME

from file_conversor.cli.pipeline.create_cmd import typer_cmd as create_cmd
from file_conversor.cli.pipeline.execute_cmd import typer_cmd as execute_cmd

_ = get_translation()

pipeline_cmd = typer.Typer(
    name=COMMAND_NAME,
    help=f"""{_('Pipeline file processing (task automation)')}

                {_('The pipeline processsing by processing an input folder, passing those files to the next pipeline stage, and processing them inside that stage. This process continues (output of the current stage is the input of the next stage), until those files reach the end of the pipeline.')}



                {_('Example')}:

                - {_('Input folder')} => {_('Stage 1')} => {_('Stage 2')} => ... => {_('Output Folder')}
        """,
)
pipeline_cmd.add_typer(create_cmd)
pipeline_cmd.add_typer(execute_cmd)



# src\file_conversor\utils\dominate_utils.py

from dominate import document as document_tag
from dominate import tags

from typing import cast


# create template tag
class template_tag(tags.html_tag):
    """Custom Dominate tag representing <template>."""
    tagname = "template"


def template(*args, **kwargs) -> template_tag:
    return cast(template_tag, template_tag(*args, **kwargs))


def document(*args, **kwargs) -> document_tag:
    return cast(document_tag, document_tag(*args, **kwargs))


def a(*args, **kwargs) -> tags.a:
    return cast(tags.a, tags.a(*args, **kwargs))


def abbr(*args, **kwargs) -> tags.abbr:
    return cast(tags.abbr, tags.abbr(*args, **kwargs))


def address(*args, **kwargs) -> tags.address:
    return cast(tags.address, tags.address(*args, **kwargs))


def area(*args, **kwargs) -> tags.area:
    return cast(tags.area, tags.area(*args, **kwargs))


def article(*args, **kwargs) -> tags.article:
    return cast(tags.article, tags.article(*args, **kwargs))


def aside(*args, **kwargs) -> tags.aside:
    return cast(tags.aside, tags.aside(*args, **kwargs))


def audio(*args, **kwargs) -> tags.audio:
    return cast(tags.audio, tags.audio(*args, **kwargs))


def b(*args, **kwargs) -> tags.b:
    return cast(tags.b, tags.b(*args, **kwargs))


def base(*args, **kwargs) -> tags.base:
    return cast(tags.base, tags.base(*args, **kwargs))


def bdi(*args, **kwargs) -> tags.bdi:
    return cast(tags.bdi, tags.bdi(*args, **kwargs))


def bdo(*args, **kwargs) -> tags.bdo:
    return cast(tags.bdo, tags.bdo(*args, **kwargs))


def blockquote(*args, **kwargs) -> tags.blockquote:
    return cast(tags.blockquote, tags.blockquote(*args, **kwargs))


def body(*args, **kwargs) -> tags.body:
    return cast(tags.body, tags.body(*args, **kwargs))


def br(*args, **kwargs) -> tags.br:
    return cast(tags.br, tags.br(*args, **kwargs))


def button(*args, **kwargs) -> tags.button:
    return cast(tags.button, tags.button(*args, **kwargs))


def canvas(*args, **kwargs) -> tags.canvas:
    return cast(tags.canvas, tags.canvas(*args, **kwargs))


def caption(*args, **kwargs) -> tags.caption:
    return cast(tags.caption, tags.caption(*args, **kwargs))


def cite(*args, **kwargs) -> tags.cite:
    return cast(tags.cite, tags.cite(*args, **kwargs))


def code(*args, **kwargs) -> tags.code:
    return cast(tags.code, tags.code(*args, **kwargs))


def col(*args, **kwargs) -> tags.col:
    return cast(tags.col, tags.col(*args, **kwargs))


def colgroup(*args, **kwargs) -> tags.colgroup:
    return cast(tags.colgroup, tags.colgroup(*args, **kwargs))


def command(*args, **kwargs) -> tags.command:
    return cast(tags.command, tags.command(*args, **kwargs))


def comment(*args, **kwargs) -> tags.comment:
    return cast(tags.comment, tags.comment(*args, **kwargs))


def datalist(*args, **kwargs) -> tags.datalist:
    return cast(tags.datalist, tags.datalist(*args, **kwargs))


def dd(*args, **kwargs) -> tags.dd:
    return cast(tags.dd, tags.dd(*args, **kwargs))


def details(*args, **kwargs) -> tags.details:
    return cast(tags.details, tags.details(*args, **kwargs))


def dfn(*args, **kwargs) -> tags.dfn:
    return cast(tags.dfn, tags.dfn(*args, **kwargs))


def div(*args, **kwargs) -> tags.div:
    return cast(tags.div, tags.div(*args, **kwargs))


def dl(*args, **kwargs) -> tags.dl:
    return cast(tags.dl, tags.dl(*args, **kwargs))


def dom1core(*args, **kwargs) -> tags.dom1core:
    return cast(tags.dom1core, tags.dom1core(*args, **kwargs))


def dom_tag(*args, **kwargs) -> tags.dom_tag:
    return cast(tags.dom_tag, tags.dom_tag(*args, **kwargs))


def dt(*args, **kwargs) -> tags.dt:
    return cast(tags.dt, tags.dt(*args, **kwargs))


def em(*args, **kwargs) -> tags.em:
    return cast(tags.em, tags.em(*args, **kwargs))


def embed(*args, **kwargs) -> tags.embed:
    return cast(tags.embed, tags.embed(*args, **kwargs))


def fieldset(*args, **kwargs) -> tags.fieldset:
    return cast(tags.fieldset, tags.fieldset(*args, **kwargs))


def figcaption(*args, **kwargs) -> tags.figcaption:
    return cast(tags.figcaption, tags.figcaption(*args, **kwargs))


def figure(*args, **kwargs) -> tags.figure:
    return cast(tags.figure, tags.figure(*args, **kwargs))


def footer(*args, **kwargs) -> tags.footer:
    return cast(tags.footer, tags.footer(*args, **kwargs))


def font(*args, **kwargs) -> tags.font:
    return cast(tags.font, tags.font(*args, **kwargs))


def form(*args, **kwargs) -> tags.form:
    return cast(tags.form, tags.form(*args, **kwargs))


def h1(*args, **kwargs) -> tags.h1:
    return cast(tags.h1, tags.h1(*args, **kwargs))


def h2(*args, **kwargs) -> tags.h2:
    return cast(tags.h2, tags.h2(*args, **kwargs))


def h3(*args, **kwargs) -> tags.h3:
    return cast(tags.h3, tags.h3(*args, **kwargs))


def h4(*args, **kwargs) -> tags.h4:
    return cast(tags.h4, tags.h4(*args, **kwargs))


def h5(*args, **kwargs) -> tags.h5:
    return cast(tags.h5, tags.h5(*args, **kwargs))


def h6(*args, **kwargs) -> tags.h6:
    return cast(tags.h6, tags.h6(*args, **kwargs))


def head(*args, **kwargs) -> tags.head:
    return cast(tags.head, tags.head(*args, **kwargs))


def header(*args, **kwargs) -> tags.header:
    return cast(tags.header, tags.header(*args, **kwargs))


def hgroup(*args, **kwargs) -> tags.hgroup:
    return cast(tags.hgroup, tags.hgroup(*args, **kwargs))


def hr(*args, **kwargs) -> tags.hr:
    return cast(tags.hr, tags.hr(*args, **kwargs))


def html(*args, **kwargs) -> tags.html:
    return cast(tags.html, tags.html(*args, **kwargs))


def html_tag(*args, **kwargs) -> tags.html_tag:
    return cast(tags.html_tag, tags.html_tag(*args, **kwargs))


def i(*args, **kwargs) -> tags.i:
    return cast(tags.i, tags.i(*args, **kwargs))


def iframe(*args, **kwargs) -> tags.iframe:
    return cast(tags.iframe, tags.iframe(*args, **kwargs))


def img(*args, **kwargs) -> tags.img:
    return cast(tags.img, tags.img(*args, **kwargs))


def input_(*args, **kwargs) -> tags.input_:
    return cast(tags.input_, tags.input_(*args, **kwargs))


def ins(*args, **kwargs) -> tags.ins:
    return cast(tags.ins, tags.ins(*args, **kwargs))


def kbd(*args, **kwargs) -> tags.kbd:
    return cast(tags.kbd, tags.kbd(*args, **kwargs))


def keygen(*args, **kwargs) -> tags.keygen:
    return cast(tags.keygen, tags.keygen(*args, **kwargs))


def label(*args, **kwargs) -> tags.label:
    return cast(tags.label, tags.label(*args, **kwargs))


def legend(*args, **kwargs) -> tags.legend:
    return cast(tags.legend, tags.legend(*args, **kwargs))


def li(*args, **kwargs) -> tags.li:
    return cast(tags.li, tags.li(*args, **kwargs))


def link(*args, **kwargs) -> tags.link:
    return cast(tags.link, tags.link(*args, **kwargs))


def main(*args, **kwargs) -> tags.main:
    return cast(tags.main, tags.main(*args, **kwargs))


def map_(*args, **kwargs) -> tags.map_:
    return cast(tags.map_, tags.map_(*args, **kwargs))


def meter(*args, **kwargs) -> tags.meter:
    return cast(tags.meter, tags.meter(*args, **kwargs))


def menu(*args, **kwargs) -> tags.menu:
    return cast(tags.menu, tags.menu(*args, **kwargs))


def mark(*args, **kwargs) -> tags.mark:
    return cast(tags.mark, tags.mark(*args, **kwargs))


def meta(*args, **kwargs) -> tags.meta:
    return cast(tags.meta, tags.meta(*args, **kwargs))


def nav(*args, **kwargs) -> tags.nav:
    return cast(tags.nav, tags.nav(*args, **kwargs))


def noscript(*args, **kwargs) -> tags.noscript:
    return cast(tags.noscript, tags.noscript(*args, **kwargs))


def object_(*args, **kwargs) -> tags.object_:
    return cast(tags.object_, tags.object_(*args, **kwargs))


def ol(*args, **kwargs) -> tags.ol:
    return cast(tags.ol, tags.ol(*args, **kwargs))


def optgroup(*args, **kwargs) -> tags.optgroup:
    return cast(tags.optgroup, tags.optgroup(*args, **kwargs))


def option(*args, **kwargs) -> tags.option:
    return cast(tags.option, tags.option(*args, **kwargs))


def output(*args, **kwargs) -> tags.output:
    return cast(tags.output, tags.output(*args, **kwargs))


def p(*args, **kwargs) -> tags.p:
    return cast(tags.p, tags.p(*args, **kwargs))


def param(*args, **kwargs) -> tags.param:
    return cast(tags.param, tags.param(*args, **kwargs))


def pre(*args, **kwargs) -> tags.pre:
    return cast(tags.pre, tags.pre(*args, **kwargs))


def progress(*args, **kwargs) -> tags.progress:
    return cast(tags.progress, tags.progress(*args, **kwargs))


def q(*args, **kwargs) -> tags.q:
    return cast(tags.q, tags.q(*args, **kwargs))


def rp(*args, **kwargs) -> tags.rp:
    return cast(tags.rp, tags.rp(*args, **kwargs))


def rt(*args, **kwargs) -> tags.rt:
    return cast(tags.rt, tags.rt(*args, **kwargs))


def ruby(*args, **kwargs) -> tags.ruby:
    return cast(tags.ruby, tags.ruby(*args, **kwargs))


def s(*args, **kwargs) -> tags.s:
    return cast(tags.s, tags.s(*args, **kwargs))


def samp(*args, **kwargs) -> tags.samp:
    return cast(tags.samp, tags.samp(*args, **kwargs))


def script(*args, **kwargs) -> tags.script:
    return cast(tags.script, tags.script(*args, **kwargs))


def section(*args, **kwargs) -> tags.section:
    return cast(tags.section, tags.section(*args, **kwargs))


def select(*args, **kwargs) -> tags.select:
    return cast(tags.select, tags.select(*args, **kwargs))


def small(*args, **kwargs) -> tags.small:
    return cast(tags.small, tags.small(*args, **kwargs))


def source(*args, **kwargs) -> tags.source:
    return cast(tags.source, tags.source(*args, **kwargs))


def span(*args, **kwargs) -> tags.span:
    return cast(tags.span, tags.span(*args, **kwargs))


def strong(*args, **kwargs) -> tags.strong:
    return cast(tags.strong, tags.strong(*args, **kwargs))


def style(*args, **kwargs) -> tags.style:
    return cast(tags.style, tags.style(*args, **kwargs))


def sub(*args, **kwargs) -> tags.sub:
    return cast(tags.sub, tags.sub(*args, **kwargs))


def summary(*args, **kwargs) -> tags.summary:
    return cast(tags.summary, tags.summary(*args, **kwargs))


def sup(*args, **kwargs) -> tags.sup:
    return cast(tags.sup, tags.sup(*args, **kwargs))


def table(*args, **kwargs) -> tags.table:
    return cast(tags.table, tags.table(*args, **kwargs))


def tbody(*args, **kwargs) -> tags.tbody:
    return cast(tags.tbody, tags.tbody(*args, **kwargs))


def td(*args, **kwargs) -> tags.td:
    return cast(tags.td, tags.td(*args, **kwargs))


def textarea(*args, **kwargs) -> tags.textarea:
    return cast(tags.textarea, tags.textarea(*args, **kwargs))


def tfoot(*args, **kwargs) -> tags.tfoot:
    return cast(tags.tfoot, tags.tfoot(*args, **kwargs))


def th(*args, **kwargs) -> tags.th:
    return cast(tags.th, tags.th(*args, **kwargs))


def thead(*args, **kwargs) -> tags.thead:
    return cast(tags.thead, tags.thead(*args, **kwargs))


def time_(*args, **kwargs) -> tags.time_:
    return cast(tags.time_, tags.time_(*args, **kwargs))


def title(*args, **kwargs) -> tags.title:
    return cast(tags.title, tags.title(*args, **kwargs))


def track(*args, **kwargs) -> tags.track:
    return cast(tags.track, tags.track(*args, **kwargs))


def tr(*args, **kwargs) -> tags.tr:
    return cast(tags.tr, tags.tr(*args, **kwargs))


def u(*args, **kwargs) -> tags.u:
    return cast(tags.u, tags.u(*args, **kwargs))


def ul(*args, **kwargs) -> tags.ul:
    return cast(tags.ul, tags.ul(*args, **kwargs))


def video(*args, **kwargs) -> tags.video:
    return cast(tags.video, tags.video(*args, **kwargs))


def wbr(*args, **kwargs) -> tags.wbr:
    return cast(tags.wbr, tags.wbr(*args, **kwargs))


def var(*args, **kwargs) -> tags.var:
    return cast(tags.var, tags.var(*args, **kwargs))


__all__ = [
    'tags',
    'document',
    'template',
    'a',
    'abbr',
    'address',
    'area',
    'article',
    'aside',
    'audio',
    'b',
    'base',
    'bdi',
    'bdo',
    'blockquote',
    'body',
    'br',
    'button',
    'canvas',
    'caption',
    'cite',
    'code',
    'col',
    'colgroup',
    'command',
    'comment',
    'datalist',
    'dd',
    'details',
    'dfn',
    'div',
    'dl',
    'dom1core',
    'dom_tag',
    'dt',
    'em',
    'embed',
    'fieldset',
    'figcaption',
    'figure',
    'footer',
    'font',
    'form',
    'h1',
    'h2',
    'h3',
    'h4',
    'h5',
    'h6',
    'head',
    'header',
    'hgroup',
    'hr',
    'html',
    'html_tag',
    'i',
    'iframe',
    'img',
    'input_',
    'ins',
    'kbd',
    'keygen',
    'label',
    'legend',
    'li',
    'link',
    'main',
    'map_',
    'meter',
    'menu',
    'mark',
    'meta',
    'nav',
    'noscript',
    'object_',
    'ol',
    'optgroup',
    'option',
    'output',
    'p',
    'param',
    'pre',
    'progress',
    'q',
    'rp',
    'rt',
    'ruby',
    's',
    'samp',
    'script',
    'section',
    'select',
    'small',
    'source',
    'span',
    'strong',
    'style',
    'sub',
    'summary',
    'sup',
    'table',
    'tbody',
    'td',
    'textarea',
    'tfoot',
    'th',
    'thead',
    'time_',
    'title',
    'track',
    'tr',
    'u',
    'ul',
    'video',
    'wbr',
    'var',
    'video',
    'wbr',
    'var'
]



# src/file_conversor/backend/gui/ebook/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui.ebook._index import ebook_index

from file_conversor.backend.gui.ebook.convert import ebook_convert


def routes():
    return [
        FlaskRoute(
            rule="/ebook",
            handler=ebook_index
        ),
        FlaskRoute(
            rule="/ebook/convert",
            handler=ebook_convert
        ),
    ]


__all__ = ['routes']




# src\file_conversor\cli\audio\convert_cmd.py

import typer

from rich import print

from typing import Annotated, List
from pathlib import Path

# user-provided modules
from file_conversor.backend import FFmpegBackend

from file_conversor.cli.audio._ffmpeg_cmd import ffmpeg_audio_run
from file_conversor.cli.audio._typer import COMMAND_NAME, CONVERT_NAME

from file_conversor.config import Environment, Configuration, State, Log, get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.validators import check_valid_options
from file_conversor.utils.typer_utils import AudioBitrateOption, AudioCodecOption, FormatOption, InputFilesArgument, OutputDirOption

from file_conversor.system.win import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = FFmpegBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    # FFMPEG commands
    icons_folder_path = Environment.get_icons_folder()
    for ext in FFmpegBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="to_m4a",
                description="To M4A",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" -f m4a "%1"',
                icon=str(icons_folder_path / 'm4a.ico'),
            ),
            WinContextCommand(
                name="to_mp3",
                description="To MP3",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" -f mp3 "%1"',
                icon=str(icons_folder_path / 'mp3.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


@typer_cmd.command(
    name=CONVERT_NAME,
    help=f"""
        {_('Convert a audio/video file to an audio format.')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.webm -f mp3 -od output_dir/ --audio-bitrate 192`
        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.mp3 -f m4a`
    """)
def convert_audio(
    input_files: Annotated[List[Path], InputFilesArgument(FFmpegBackend.SUPPORTED_IN_FORMATS)],

    file_format: Annotated[str, FormatOption(FFmpegBackend.SUPPORTED_OUT_AUDIO_FORMATS)],

    audio_bitrate: Annotated[int, AudioBitrateOption()] = CONFIG["audio-bitrate"],

    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    ffmpeg_audio_run(
        input_files,
        file_format=file_format,
        audio_bitrate=audio_bitrate,
        output_dir=output_dir,
    )




# src\file_conversor\cli\audio\info_cmd.py

import typer

from rich import print

from typing import Annotated, List
from datetime import timedelta
from pathlib import Path

from rich import print
from rich.text import Text
from rich.panel import Panel
from rich.console import Group

# user-provided modules
from file_conversor.backend import FFprobeBackend
from file_conversor.cli.audio._typer import COMMAND_NAME, INFO_NAME

from file_conversor.config import Environment, Configuration, State, Log, get_translation

from file_conversor.utils.backend import FFprobeParser
from file_conversor.utils.formatters import format_bytes, format_bitrate
from file_conversor.utils.typer_utils import InputFilesArgument

from file_conversor.system.win import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = FFprobeBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    # FFMPEG commands
    icons_folder_path = Environment.get_icons_folder()
    for ext in FFprobeBackend.SUPPORTED_IN_AUDIO_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="info",
                description="Get Info",
                command=f'cmd /k "{Environment.get_executable()} "{COMMAND_NAME}" "{INFO_NAME}" "%1""',
                icon=str(icons_folder_path / 'info.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


@typer_cmd.command(
    name=INFO_NAME,
    help=f"""
        {_('Get information about a audio file.')}

        {_('This command retrieves metadata and other information about the audio file')}:

        - {_('Format')} (mp3, m4a, etc)

        - {_('Duration')} (HH:MM:SS)

        - {_('Other properties')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {INFO_NAME} filename.m4a`

        - `file_conversor {COMMAND_NAME} {INFO_NAME} other_filename.mp3`
    """)
def info(
    input_files: Annotated[List[Path], InputFilesArgument(FFprobeBackend.SUPPORTED_IN_AUDIO_FORMATS)],
):

    ffprobe_backend = FFprobeBackend(
        install_deps=CONFIG['install-deps'],
        verbose=STATE["verbose"],
    )
    for filename in input_files:
        logger.info(f"{_('Parsing file metadata for')} '{filename}' ...")
        try:
            parser = FFprobeParser(ffprobe_backend, filename)
            parser.run()
            # Agrupar e exibir tudo com Rich
            group = Group(*[
                *parser.get_format().rich(),
                *parser.get_streams().rich(),
                *parser.get_chapters().rich(),
            ])
            print(Panel(group, title=f"🧾 {_('File Analysis')}", border_style="blue"))
        except Exception as e:
            logger.error(f"{_('Error parsing file')} '{filename}': {e}")
            continue



# src\file_conversor\backend\gui\_components\footer.py

from file_conversor.utils.dominate_utils import *


def Footer(**kwargs):
    """
    Create a footer component with project and license information.
    """
    with div(_id="footer", _class="content has-text-centered", **kwargs) as footer:
        # First line
        with div():
            with strong():
                a(
                    "File Conversor",
                    _href="https://github.com/andre-romano/file_conversor",
                    _target="_blank"
                )
            span(" by ")
            a("Andre Madureira", _href="https://github.com/andre-romano", _target="_blank")

        # Second line (license info)
        with div():
            span("The source code is licensed under the ")
            a(
                "Apache-2.0",
                _href="https://github.com/andre-romano/file_conversor/blob/master/LICENSE",
                _target="_blank"
            )
            span(" license.")
    return footer


__all__ = ["Footer"]



# src\file_conversor\backend\compress_backend.py

from pathlib import Path
from typing import Any

# user-provided imports
from file_conversor.config import Log
from file_conversor.config.locale import get_translation

from file_conversor.backend.abstract_backend import AbstractBackend

from file_conversor.backend.image._gifsicle_backend import _GifSicleBackend
from file_conversor.backend.image._mozjpeg_backend import _MozJPEGBackend
from file_conversor.backend.image._oxipng_backend import _OxiPNGBackend

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class CompressBackend(AbstractBackend):
    """
    Provides an interface for handling files using mozjpeg.
    """

    SUPPORTED_IN_FORMATS: dict[str, dict[str, type]] = {
        "gif": {
            "cls": _GifSicleBackend
        },
        "jpg": {
            "cls": _MozJPEGBackend
        },
        "jpeg": {
            "cls": _MozJPEGBackend
        },
        "png": {
            "cls": _OxiPNGBackend
        },
    }
    SUPPORTED_OUT_FORMATS = SUPPORTED_IN_FORMATS
    EXTERNAL_DEPENDENCIES = {
        *_GifSicleBackend.EXTERNAL_DEPENDENCIES,
        *_MozJPEGBackend.EXTERNAL_DEPENDENCIES,
        *_OxiPNGBackend.EXTERNAL_DEPENDENCIES,
    }

    def __init__(
        self,
        install_deps: bool | None,
        verbose: bool = False,
    ):
        """
        Initialize the backend.

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 
        :param verbose: Verbose logging. Defaults to False.      

        :raises RuntimeError: if dependency is not found
        """
        super().__init__()
        # get required dependencies
        for _, opts in CompressBackend.SUPPORTED_IN_FORMATS.items():
            cls = opts["cls"]
            cls(
                install_deps=install_deps,
                verbose=verbose,
            )
        self._install_deps = install_deps
        self._verbose = verbose

    def compress(
        self,
            input_file: str | Path,
            output_file: str | Path,
            **kwargs,
    ):
        """
        Execute the command to compress the input file.

        :param input_file: Input file path.
        :param output_file: Output file path.      
        :param kwargs: Arguments.

        :return: Subprocess.CompletedProcess object

        :raises RuntimeError: If backend encounters an error during execution.
        """
        # Execute the command
        output_file = Path(output_file)
        out_ext = output_file.suffix[1:]

        out_opts = CompressBackend.SUPPORTED_OUT_FORMATS[out_ext]
        BACKEND_CLASS = out_opts["cls"]

        backend = BACKEND_CLASS(
            install_deps=self._install_deps,
            verbose=self._verbose,
        )
        return backend.compress(
            input_file=input_file,
            output_file=output_file,
            **kwargs,
        )



# src/file_conversor/utils/backend/ffprobe_utils.py

import subprocess

from datetime import timedelta
from pathlib import Path

from rich import print
from rich.text import Text
from rich.panel import Panel
from rich.console import Group

# user-provided imports
from file_conversor.backend.audio_video import FFprobeBackend

from file_conversor.utils.bulma_utils import *
from file_conversor.utils.dominate_utils import *
from file_conversor.utils.formatters import format_bitrate, format_bytes

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


class _FFprobeFormatInfo:
    def __init__(self, input_file: Path, metadata: dict) -> None:
        super().__init__()
        self.input_file = input_file
        self.format_info = metadata.get("format", {})

    def _parse(self):
        duration = self.format_info.get('duration', 'N/A')
        if duration != "N/A":
            duration_secs = int(float(duration))
            duration_td = timedelta(seconds=duration_secs)
            duration = str(duration_td)

        size = self.format_info.get("size", "N/A")
        if size != "N/A":
            size = format_bytes(float(size))

        bitrate = self.format_info.get('bit_rate', 'N/A')
        if bitrate != "N/A":
            bitrate = format_bitrate(int(bitrate))

        format_name = self.format_info.get('format_name', 'N/A')
        return duration, size, bitrate, format_name

    def rich(self):
        duration, size, bitrate, format_name = self._parse()
        return [
            Text(f"📁 {_('File Information')}:", style="bold cyan"),
            f"  - {_('Name')}: {self.input_file.name}",
            f"  - {_('Format')}: {format_name}",
            f"  - {_('Duration')}: {duration}",
            f"  - {_('Size')}: {size}",
            f"  - {_('Bitrate')}: {bitrate}",
        ]

    def div(self):
        duration, size, bitrate, format_name = self._parse()
        with div() as result:
            div(f"{_('File Information')}:")
            div(f"  - {_('Name')}: {self.input_file.name}")
            div(f"  - {_('Format')}: {format_name}")
            div(f"  - {_('Duration')}: {duration}")
            div(f"  - {_('Size')}: {size}")
            div(f"  - {_('Bitrate')}: {bitrate}")
            br()
        return result


class _FFprobeStreamsInfo:
    def __init__(self, input_file: Path, metadata: dict) -> None:
        super().__init__()
        self.input_file = input_file
        self.streams_info = metadata.get("streams", [])

    def _parse(self, stream: dict):
        stream_type = str(stream.get("codec_type", "unknown")).upper()
        codec = stream.get("codec_name", "N/A")
        resolution = f"{stream.get('width', '?')}x{stream.get('height', '?')}" if stream_type == "video" else ""
        bitrate = stream.get("bit_rate", "N/A")

        if bitrate != "N/A":
            bitrate = format_bitrate(int(bitrate))

        sample_rate = f"{stream.get('sample_rate', 'N/A')} Hz" if stream_type == "AUDIO" else ""
        channels = stream.get('channels', 'N/A') if stream_type == "AUDIO" else ""
        return stream_type, codec, resolution, bitrate, sample_rate, channels

    def rich(self):
        formatted = []
        if self.streams_info:
            formatted.append(Text(f"\n🎬 {_("Media Streams")}:", style="bold yellow"))
        for i, stream in enumerate(self.streams_info):
            stream_type, codec, resolution, bitrate, sample_rate, channels = self._parse(stream)

            formatted.append(f"\n  🔹 {_('Stream')} #{i} ({stream_type}):")
            formatted.append(f"    - {_('Codec')}: {codec}")
            if resolution:
                formatted.append(f"    - {_('Resolution')}: {resolution}")
            formatted.append(f"    - {_('Bitrate')}: {bitrate}")
            if stream_type == "audio":
                formatted.append(f"    - {_('Sampling rate')}: {sample_rate} Hz")
                formatted.append(f"    - {_('Channels')}: {channels}")
        return formatted

    def div(self):
        with div() as result:
            if self.streams_info:
                div(f"{_("Media Streams")}:")
            for i, stream in enumerate(self.streams_info):
                stream_type, codec, resolution, bitrate, sample_rate, channels = self._parse(stream)

                div(f"    {_('Stream')} #{i} ({stream_type.upper()}):")
                div(f"    - {_('Codec')}: {codec}")
                if resolution:
                    div(f"    - {_('Resolution')}: {resolution}")
                div(f"    - {_('Bitrate')}: {bitrate}")
                if stream_type == "audio":
                    div(f"    - {_('Sampling rate')}: {sample_rate}")
                    div(f"    - {_('Channels')}: {channels}")
                br()
            br()
        return result


class _FFprobeChaptersInfo:
    def __init__(self, input_file: Path, metadata: dict) -> None:
        super().__init__()
        self.input_file = input_file
        self.chapters_info = metadata.get("chapters", [])

    def _parse(self, chapter: dict):
        title = chapter.get('tags', {}).get('title', 'N/A')
        start = f"{chapter.get('start_time', 'N/A')}s"
        return title, start

    def rich(self):
        formatted = []
        if self.chapters_info:
            formatted.append(Text(f"\n📖 {_('Chapters')}:", style="bold green"))
        for chapter in self.chapters_info:
            title, start = self._parse(chapter)
            formatted.append(f"  - {title} ({_('Time')}: {start})")
        return formatted

    def div(self):
        with div() as result:
            if self.chapters_info:
                div(f"{_('Chapters')}:")
            for chapter in self.chapters_info:
                title, start = self._parse(chapter)
                div(f"  - {title} ({_('Time')}: {start})")
        return result


class FFprobeParser:
    def __init__(self, backend: FFprobeBackend, input_file: Path) -> None:
        super().__init__()
        self.input_file = input_file
        self.backend = backend

    def run(self):
        try:
            self.metadata = self.backend.info(self.input_file)
        except subprocess.CalledProcessError:
            raise RuntimeError(f"{_('File')} '{self.input_file}' {_('is corrupted or has inconsistencies')}")

    def get_format(self):
        return _FFprobeFormatInfo(self.input_file, self.metadata)

    def get_streams(self):
        return _FFprobeStreamsInfo(self.input_file, self.metadata)

    def get_chapters(self):
        return _FFprobeChaptersInfo(self.input_file, self.metadata)


__all__ = [
    "FFprobeParser",
]



# src/file_conversor/backend/gui/_api/pdf/decrypt.py

from flask import json, render_template, request, url_for
from pathlib import Path
from typing import Any

# user-provided modules
from file_conversor.backend.gui.flask_api import FlaskApi
from file_conversor.backend.gui.flask_api_status import FlaskApiStatus

from file_conversor.backend.pdf import PyPDFBackend

from file_conversor.cli.pdf.decrypt_cmd import execute_pdf_decrypt_cmd
from file_conversor.utils import CommandManager, ProgressManager

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def _api_thread(params: dict[str, Any], status: FlaskApiStatus) -> None:
    """Thread to handle PDF decryption."""
    logger.debug(f"PDF decryption thread received: {params}")
    input_files: list[Path] = [Path(i) for i in params['input-files']]
    output_dir: Path = Path(params['output-dir'])
    password: str = str(params['password'])

    execute_pdf_decrypt_cmd(
        input_files=input_files,
        password=password,
        output_dir=output_dir,
        progress_callback=status.set_progress,
    )


def api_pdf_decrypt():
    """API endpoint to decrypt PDF documents."""
    logger.info(f"[bold]{_('PDF decryption requested via API.')}[/]")
    return FlaskApi.execute_response(_api_thread)



# src\file_conversor\cli\xls\_info.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

# command
COMMAND_NAME = "xls"

# SUBCOMMANDS
CONVERT_NAME = "convert"



# src\file_conversor\utils\dominate_bulma\__init__.py

from file_conversor.utils.dominate_bulma.breadcrumb import *
from file_conversor.utils.dominate_bulma.card import *
from file_conversor.utils.dominate_bulma.class_ import *
from file_conversor.utils.dominate_bulma.figure import *
from file_conversor.utils.dominate_bulma.flex import *
from file_conversor.utils.dominate_bulma.font_awesome_icon import *
from file_conversor.utils.dominate_bulma.footer import *
from file_conversor.utils.dominate_bulma.form_button import *
from file_conversor.utils.dominate_bulma.form_checkbox import *
from file_conversor.utils.dominate_bulma.form_field import *
from file_conversor.utils.dominate_bulma.form_file_list import *
from file_conversor.utils.dominate_bulma.form_input import *
from file_conversor.utils.dominate_bulma.form_output_dir import *
from file_conversor.utils.dominate_bulma.form_output_file import *
from file_conversor.utils.dominate_bulma.form_radio import *
from file_conversor.utils.dominate_bulma.form_select import *
from file_conversor.utils.dominate_bulma.form_textarea import *
from file_conversor.utils.dominate_bulma.form import *
from file_conversor.utils.dominate_bulma.grid import *
from file_conversor.utils.dominate_bulma.level import *
from file_conversor.utils.dominate_bulma.margin import *
from file_conversor.utils.dominate_bulma.media import *
from file_conversor.utils.dominate_bulma.modal import *
from file_conversor.utils.dominate_bulma.navbar import *
from file_conversor.utils.dominate_bulma.padding import *

# pages
from file_conversor.utils.dominate_bulma.page_base import *
from file_conversor.utils.dominate_bulma.page_card_grid import *
from file_conversor.utils.dominate_bulma.page_form import *

from file_conversor.utils.dominate_bulma.progress import *
from file_conversor.utils.dominate_bulma.status_bar import *
from file_conversor.utils.dominate_bulma.tabs import *
from file_conversor.utils.dominate_bulma.text import *
from file_conversor.utils.dominate_bulma.typography import *
from file_conversor.utils.dominate_bulma.visibility import *



# src\file_conversor\backend\audio_video\ffmpeg_backend.py

"""
This module provides functionalities for handling audio and video files using FFmpeg.
"""

import subprocess
import re

from pathlib import Path
from typing import Any, Callable, Iterable

# user-provided imports
from file_conversor.backend.audio_video.abstract_ffmpeg_backend import AbstractFFmpegBackend
from file_conversor.backend.audio_video.ffprobe_backend import FFprobeBackend

from file_conversor.backend.audio_video.ffmpeg_filter import FFmpegFilter
from file_conversor.backend.audio_video.ffmpeg_codec import FFmpegAudioCodec, FFmpegVideoCodec
from file_conversor.backend.audio_video.format_container import FormatContainer

from file_conversor.config import Environment, Log, get_translation

from file_conversor.utils.validators import check_file_format
from file_conversor.utils.command_manager import CommandManager

from file_conversor.system import is_windows

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class FFmpegBackend(AbstractFFmpegBackend):
    """
    FFmpegBackend is a class that provides an interface for handling audio and video files using FFmpeg.
    """

    EXTERNAL_DEPENDENCIES = set([
        "ffmpeg",
    ])

    @staticmethod
    def _clean_two_pass_log_file(logfile: Path):
        for filepath in logfile.parent.glob(logfile.name + "-0.log*"):
            try:
                if not filepath.exists():
                    continue
                filepath.unlink()
            except Exception as e:
                logger.warning(f"Failed to remove log file '{filepath}': {e}")

    def __init__(
        self,
        install_deps: bool | None,
        verbose: bool = False,
        overwrite_output: bool = False,
        stats: bool = False,
    ):
        """
        Initialize the FFMpeg backend.

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 
        :param verbose: Verbose logging. Defaults to False.      

        :raises RuntimeError: if ffmpeg dependency is not found
        """
        super().__init__(
            install_deps=install_deps,
            verbose=verbose,
        )
        self._stats = stats
        self._overwrite_output = overwrite_output

        self._input_file: Path | None = None
        self._output_file: Path | None = None

        self._audio_bitrate: int | None = None
        self._video_bitrate: int | None = None

        self._global_options: list[str] = []
        self._in_opts: list[str] = []
        self._out_opts: list[str] = []

        self._out_container: FormatContainer | None = None

        self._progress_callback: Callable[[float], Any] | None = None

    def _execute_progress_callback(
        self,
        process: subprocess.Popen,
    ):
        """returns output lines read"""
        lines: list[str] = []
        PROGRESS_RE = re.compile(r'time=(\d+):(\d+):([\d\.]+)')

        ffprobe_backend = FFprobeBackend(install_deps=self._install_deps, verbose=self._verbose)
        if not self._input_file:
            raise RuntimeError(f"{_('Input file not set')}")

        file_duration_secs = ffprobe_backend.get_duration(self._input_file)
        while process.poll() is None:
            if not process.stdout:
                continue

            line = process.stdout.readline()
            match = PROGRESS_RE.search(line)
            if not match:
                lines.append(line)
                continue
            hours = int(match.group(1))
            minutes = int(match.group(2))
            seconds = float(match.group(3))

            current_time = hours * 3600 + minutes * 60 + seconds
            progress = 100.0 * (float(current_time) / file_duration_secs)
            if self._progress_callback:
                self._progress_callback(progress)
        return lines

    def _set_global_options(self):
        """Set default global options"""
        global_options = {}
        global_options["-y" if self._overwrite_output else "-n"] = ""
        if self._verbose:
            global_options["-v"] = ""  # verbose output
        if self._stats:
            global_options["-stats"] = ""  # print progress stats

        # create global options list
        self._global_options = []
        for k, v in global_options.items():
            self._global_options.extend([str(k), str(v)])

    def _set_input_file(self, input_file: str | Path):
        """
        Set the input file and check if it has a supported format.

        :param input_file: Input file path.

        :raises FileNotFoundError: If the input file does not exist.
        :raises ValueError: If the input file format is not supported.
        """
        self._in_opts = []

        # check file is found
        self._input_file = Path(input_file).resolve()
        if not self._input_file.exists() and not self._input_file.is_file():
            raise FileNotFoundError(f"Input file '{input_file}' not found")

        # check if the input file has a supported format
        check_file_format(self._input_file, self.SUPPORTED_IN_FORMATS)

        # set the input format options based on the file extension
        in_ext = self._input_file.suffix[1:]
        for k, v in self.SUPPORTED_IN_FORMATS[in_ext].items():
            self._in_opts.extend([str(k), str(v)])

    def _set_output_file(
            self,
            output_file: str | Path,
    ):
        """
        Set the output file and check if it has a supported format.

        :param output_file: Output file path.

        :raises typer.BadParameter: Unsupported format.
        """
        self._out_opts = []

        # create out dir (if it does not exists)
        self._output_file = Path(output_file).resolve()
        if self._output_file.name == "-":
            logger.warning("Null container selected. No output file will be created.")
            out_ext = "null"
        else:
            self._output_file.parent.mkdir(parents=True, exist_ok=True)

            # check if the output file has a supported format
            check_file_format(self._output_file, self.SUPPORTED_OUT_FORMATS)

            # set the output format options based on the file extension
            out_ext = self._output_file.suffix[1:]
        args, kwargs = self.SUPPORTED_OUT_FORMATS[out_ext]
        self._out_container = FormatContainer(*args, **kwargs)

    def set_files(self, input_file: str | Path, output_file: str | Path):
        """
        Set input/output files, and default global options

        :param input_file: Input file path.
        :param output_file: Output file path.      
        """
        self._set_global_options()
        self._set_input_file(input_file)
        self._set_output_file(output_file)

    def _set_codec(
            self,
            is_audio: bool,
            codec: str | None = None,
            bitrate: int | None = None,
            filters: FFmpegFilter | Iterable[FFmpegFilter] | None = None,
    ):
        if not self._out_container:
            raise RuntimeError(f"{_('Output container not set')}")

        if codec:
            if is_audio:
                self._out_container.audio_codec = FFmpegAudioCodec.from_str(codec)
            else:
                self._out_container.video_codec = FFmpegVideoCodec.from_str(codec)

        codec_obj = self._out_container.audio_codec if is_audio else self._out_container.video_codec

        if bitrate is not None and bitrate > 0:
            if is_audio:
                self._audio_bitrate = bitrate
            else:
                self._video_bitrate = bitrate
            codec_obj.set_bitrate(bitrate)

        if filters:
            if isinstance(filters, FFmpegFilter):
                filters = [filters]
            codec_obj.set_filters(*filters)

    def set_audio_codec(
        self,
        codec: str | None = None,
        bitrate: int | None = None,
        filters: FFmpegFilter | Iterable[FFmpegFilter] | None = None,
    ):
        """
        Set audio codec and bitrate

        :param codec: Codec to use. Defaults to None (use container default codec).      
        :param bitrate: Bitrate to use (in kbps). Defaults to None (use FFmpeg defaults).      
        :param filters: Filters to use. Defaults to None (do not use any filter).      

        :raises RuntimeErrors: if output container not set
        """
        self._set_codec(is_audio=True, codec=codec, bitrate=bitrate, filters=filters)

    def set_video_codec(
        self,
        codec: str | None = None,
        bitrate: int | None = None,
        filters: FFmpegFilter | Iterable[FFmpegFilter] | None = None,
        encoding_speed: str | None = None,
        quality_setting: str | None = None,
    ):
        """
        Seet video codec and bitrate

        :param codec: Codec to use. Defaults to None (use container default codec).      
        :param bitrate: Bitrate to use (in kbps). Defaults to None (use FFmpeg defaults).      
        :param filters: Filters to use. Defaults to None (do not use any filter).      
        :param encoding_speed: Encoding speed to use. Defaults to None (use codec default speed).      
        :param quality_setting: Quality setting to use. Defaults to None (use codec default quality).

        :raises RuntimeErrors: if output container not set
        """
        self._set_codec(is_audio=False, codec=codec, bitrate=bitrate, filters=filters)
        if not self._out_container:
            raise RuntimeError(f"{_('Output container not set')}")

        if encoding_speed:
            self._out_container.video_codec.set_encoding_speed(encoding_speed)

        if (not bitrate or bitrate == 0) and quality_setting:
            self._out_container.video_codec.set_quality_setting(quality_setting)

    def _execute(self):
        # build ffmpeg command
        ffmpeg_command = []
        ffmpeg_command.extend([str(self._ffmpeg_bin)])  # ffmpeg CLI
        ffmpeg_command.extend(self._global_options)    # set global options
        ffmpeg_command.extend(self._in_opts)           # set in options
        ffmpeg_command.extend(["-i", str(self._input_file)])   # set input
        ffmpeg_command.extend(self._out_opts)          # set out options
        ffmpeg_command.extend([str(self._output_file)])        # set output

        # remove empty strings
        ffmpeg_command = [arg for arg in ffmpeg_command if arg != ""]

        # Execute the FFmpeg command
        process = Environment.run_nowait(
            *ffmpeg_command,
        )

        out_lines = self._execute_progress_callback(
            process=process,
        )

        Environment.check_returncode(process, out_lines=out_lines)
        return process

    def execute(
        self,
        progress_callback: Callable[[float], Any] | None = None,
        pass_num: int = 0,
        out_opts: list[str] | None = None,
    ):
        """
        Execute the FFmpeg command to convert the input file to the output file.

        :param pass_num: Pass number for multi-pass encoding (0 for single pass, 1 for first pass, 2 for second pass). Defaults to 0.
        :param out_opts: FFmpeg custom out options. Defaults to None.

        :return: Subprocess.Popen object

        :raises RuntimeError: If FFmpeg encounters an error during execution.
        """
        self._progress_callback = progress_callback

        if pass_num not in (0, 1, 2):
            raise ValueError(f"{_('Invalid number of passes:')} {pass_num}. {_('Must be 0 (single-pass), 1 (first pass) or 2 (second pass).')}")

        if not self._input_file or not self._output_file:
            raise RuntimeError(f"{_('Input/output files not set')}")

        if not self._out_container:
            raise RuntimeError(f"{_('Output container not set')}")

        self._out_opts = []

        logdir = self._output_file.parent

        logfile = logdir / CommandManager.get_output_file(self._output_file, stem="-ffmpeg2pass", suffix="")

        if pass_num > 0:
            logger.debug(f"{_('Temporary 2-pass log file')}: {logfile}")
            # add 2-pass encoding options
            self._out_opts.extend([
                "-pass", str(pass_num),
                "-passlogfile", str(logfile),
            ])
            if self._audio_bitrate and self._audio_bitrate <= 0:
                raise ValueError(f"{_('Audio bitrate cannot be 0 when using two-pass mode.')}")
            if self._video_bitrate and self._video_bitrate <= 0:
                raise ValueError(f"{_('Video bitrate cannot be 0 when using two-pass mode.')}")

        self._out_opts.extend(self._out_container.get_options())
        self._out_opts.extend(out_opts if out_opts else [])

        original_output_file = self._output_file
        if pass_num == 1:
            self._output_file = Path("NUL" if is_windows() else "/dev/null")

        try:
            process = self._execute()
        except:
            self._clean_two_pass_log_file(logfile)
            raise

        self._output_file = original_output_file

        if pass_num in (0, 2):
            self._clean_two_pass_log_file(logfile)



# src\file_conversor\backend\text_backend.py

import toml
import json
import yaml
import xmltodict
import configparser

from typing import Any, Callable, Iterable, Sequence

from pathlib import Path

from rich import print

# user-provided imports
from file_conversor.backend.abstract_backend import AbstractBackend

from file_conversor.config import Environment, Configuration, Log
from file_conversor.config.locale import get_translation

# get app config
CONFIG = Configuration.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class AbstractTextFile:
    def __init__(self, filename: str | Path) -> None:
        super().__init__()
        self._filepath = Path(filename)

    def read(self) -> Any:
        raise NotImplementedError("not implemented")

    def write(self, data: Any):
        raise NotImplementedError("not implemented")

    def minify(self, data: Any):
        self.write(data)


class XMLTextFile(AbstractTextFile):
    def read(self):
        return xmltodict.parse(self._filepath.read_bytes())

    def write(self, data: Any, **kwargs):
        xml_str = xmltodict.unparse(data, pretty=True)
        self._filepath.write_text(xml_str, encoding="utf-8")

    def minify(self, data: Any):
        xml_str = xmltodict.unparse(data, pretty=False)
        self._filepath.write_text(xml_str, encoding="utf-8")


class JSONTextFile(AbstractTextFile):
    def read(self):
        return json.loads(self._filepath.read_bytes())

    def write(self, data: Any):
        json_str = json.dumps(data, indent=4)
        self._filepath.write_text(json_str, encoding="utf-8")

    def minify(self, data: Any):
        json_str = json.dumps(data, separators=(',', ':'), indent=None)
        self._filepath.write_text(json_str, encoding="utf-8")


class YAMLTextFile(AbstractTextFile):
    def read(self):
        with open(self._filepath, mode="r") as fp:
            return yaml.safe_load(fp)

    def write(self, data: Any):
        with open(self._filepath, mode="w") as fp:
            yaml.dump(data, fp, indent=2)

    def minify(self, data: Any):
        with open(self._filepath, mode="w") as fp:
            yaml.dump(
                data,
                fp,
                default_flow_style=True,  # forces inline compact form
                allow_unicode=True,       # preserves UTF-8 chars
            )


class TOMLTextFile(AbstractTextFile):
    def read(self):
        with open(self._filepath, mode="r") as fp:
            return toml.load(fp)

    def write(self, data: Any):
        with open(self._filepath, mode="w") as fp:
            toml.dump(data, fp)


class INITextFile(AbstractTextFile):
    def read(self):
        config = configparser.ConfigParser()
        config.read(self._filepath)
        return {section: dict(config[section]) for section in config.sections()}

    def write(self, data: Any):
        if not isinstance(data, dict):
            raise ValueError(f"Cannot convert '{self._filepath}' => INI file. Expected input format: {{section_name: {{key1: value1, key2: value2, ...}} }}.")
        config = configparser.ConfigParser()
        for section, values in data.items():
            if not isinstance(values, dict):
                raise ValueError(f"Cannot convert '{self._filepath}' => INI file. Expected input format: {{section_name: {{key1: value1, key2: value2, ...}} }}")
            config[section] = {str(k): str(v) for k, v in values.items()}
        with open(self._filepath, "w") as f:
            config.write(f)


class TextBackend(AbstractBackend):
    SUPPORTED_IN_FORMATS = {
        "json": {
            "cls": JSONTextFile,
        },
        "xml": {
            "cls": XMLTextFile,
        },
        "yaml": {
            "cls": YAMLTextFile,
        },
        "toml": {
            "cls": TOMLTextFile,
        },
        "ini": {
            "cls": INITextFile,
        },
    }
    SUPPORTED_OUT_FORMATS = {
        "json": {
            "cls": JSONTextFile,
        },
        "xml": {
            "cls": XMLTextFile,
        },
        "yaml": {
            "cls": YAMLTextFile,
        },
        "toml": {
            "cls": TOMLTextFile,
        },
        "ini": {
            "cls": INITextFile,
        },
    }
    EXTERNAL_DEPENDENCIES = set([])

    def __init__(
        self,
        verbose: bool = False,
    ):
        """
        Initialize the Batch backend.
        """
        super().__init__()
        self._verbose = verbose

    def _get_text_file(
            self,
            filepath: Path,
            formats: dict[str, dict[str, type]],
    ) -> tuple[str, AbstractTextFile]:
        """"Checks file format and returns (ext, textfile)"""
        ext = filepath.suffix[1:]
        if ext not in formats:
            raise ValueError(f"Unsupported input format '{ext}'. Supported formats: {', '.join(formats)}")
        if "cls" not in formats[ext]:
            raise ValueError(f"Cannot find class for format '{ext}'.")
        cls = formats[ext]["cls"]
        text_file = cls(filepath)
        # logger.debug(f"Filepath: {filepath} - Class: {type(text_file)}")
        return (ext, text_file)

    def convert(
            self,
            input_file: str | Path,
            output_file: str | Path,
    ):
        """
        Convert text file to other formats

        :param input_file: Input file
        :param output_file: Output file        
        """
        input_file = Path(input_file)
        output_file = Path(output_file)

        _, in_txt_file = self._get_text_file(input_file, self.SUPPORTED_IN_FORMATS)
        _, out_txt_file = self._get_text_file(output_file, self.SUPPORTED_OUT_FORMATS)

        data = in_txt_file.read()
        out_txt_file.write(data)

    def check(self,
              input_file: str | Path,
              ):
        """
        Checks if file is wellformed (structure is correct)

        :raises Exception: if file is not well structured
        """
        input_file = Path(input_file)

        _, in_txt_file = self._get_text_file(input_file, self.SUPPORTED_IN_FORMATS)
        try:
            in_txt_file.read()
        except:
            logger.error(rf"'{input_file}': [bold red]FAILED[/]")
            raise
        logger.info(rf"'{input_file}': [bold green]OK[/]")

    def minify(self,
               input_file: str | Path,
               output_file: str | Path,
               ):
        """
        Minifies text file

        :param input_file: Input file
        :param output_file: Output file  
        """
        input_file = Path(input_file)
        output_file = Path(output_file)

        _, in_txt_file = self._get_text_file(input_file, self.SUPPORTED_IN_FORMATS)
        _, out_txt_file = self._get_text_file(output_file, self.SUPPORTED_OUT_FORMATS)

        data = in_txt_file.read()
        out_txt_file.minify(data)



# src\file_conversor\utils\bulma\form_radio.py

from typing import Any

from file_conversor.utils.dominate_bulma.form_field import FormField

from file_conversor.utils.dominate_utils import *


def Radio(
    label_text: str,
    _name: str,
    _value: str,
    _class: str = "",
    **kwargs,
):
    """
    Create a radio input.

    :param label_text: The text for the radio label.
    :param _name: The name attribute for the radio input.
    :param _value: The value attribute for the radio input.
    :param _class: Additional CSS classes for the radio.
    """
    with label(_class=f"radio {_class}") as radio_label:
        input_(_type="radio", _name=_name, _value=_value, **kwargs)
        span(label_text)
    return radio_label


def FormFieldRadio(
    *options: tuple[str, str],
    _name: str,
    current_value: str,
    help: str,
    label_text: str = "",
    x_data: str = "",
    x_init: str = "",
    **kwargs,
):
    """
    Create a form field with a label and a select element.

    :param options: A list of tuples where each tuple contains (value, display_text).
    :param _name: The name attribute for the select element.
    :param current_value: The current selected value.
    :param label_text: The text for the label.
    :param help: Optional help text.
    :param x_data: Additional Alpine.js x-data properties.

    :return: The FormField element.
    """
    field = FormField(
        *[
            Radio(
                _class="is-flex is-flex-grow-1",
                _name=_name,
                _value=value,
                _title=help,
                label_text=display_text,
                **{
                    ':class': """{
                        'is-danger': !isValid,
                        'is-success': isValid,
                    }""",
                    'x-model': 'value',
                },
                **kwargs,
            )
            for value, display_text in options
        ],
        _class_control="is-flex is-flex-direction-column is-flex-grow-1",
        current_value=current_value,
        label_text=label_text,
        help=help,
        x_data=x_data,
        x_init=x_init,
    )
    return field


__all__ = [
    "Radio",
    "FormFieldRadio",
]



# src/file_conversor/backend/gui/ppt/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui.ppt._index import ppt_index

from file_conversor.backend.gui.ppt.convert import ppt_convert


def routes():
    return [
        FlaskRoute(
            rule="/ppt",
            handler=ppt_index,
        ),
        # TOOLS
        FlaskRoute(
            rule="/ppt/convert",
            handler=ppt_convert,
        ),
    ]


__all__ = ['routes']




# src\file_conversor\system\win\utils.py

import os
import platform
import subprocess
import time

from pathlib import Path

# Import winreg only on Windows to avoid ImportError on other OSes
if platform.system() == "Windows":
    import winreg
    import ctypes
else:
    # Placeholder so the names exists
    winreg = None
    ctypes = None


def _get_window_hwnd(window_title: str) -> int:
    """
    Get the window handle (HWND) for a given window title.

    :param window_title: The title of the window.
    :return: The HWND of the window.
    :raises RuntimeError: If the window is not found or ctypes is unavailable.
    """
    if ctypes is None:
        raise RuntimeError(f"ctypes is not available on this platform ({platform.system()}).")

    hwnd = ctypes.windll.user32.FindWindowW(None, window_title)
    if hwnd == 0:
        raise RuntimeError(f"user32.FindWindowW: Window with title '{window_title}' not found.")
    return hwnd


def is_admin() -> bool:
    """True if app running with admin priviledges, False otherwise."""
    try:
        if ctypes:
            return ctypes.windll.shell32.IsUserAnAdmin()  # pyright: ignore[reportAttributeAccessIssue]
    except:
        pass
    return False


def reload_user_path():
    """Reload user PATH in current process."""
    if winreg is None:
        return
    with winreg.OpenKey(winreg.HKEY_CURRENT_USER, r"Environment") as key:  # pyright: ignore[reportAttributeAccessIssue]
        user_path, _ = winreg.QueryValueEx(key, "PATH")  # pyright: ignore[reportAttributeAccessIssue]
        os.environ["PATH"] = user_path + os.pathsep + os.environ["PATH"]


def restart_explorer():
    # Step 1: kill explorer.exe
    subprocess.run(
        ["taskkill", "/f", "/im", "explorer.exe"],
        capture_output=True,
        text=True,  # Capture output as text (Python 3.7+)
        check=True,
    )
    # Wait briefly to ensure process termination
    time.sleep(0.5)  # Increased delay for stability
    # Step 2: Restart explorer.exe
    subprocess.Popen(
        "explorer.exe",
        shell=True,
        stdin=subprocess.DEVNULL,
        stdout=subprocess.DEVNULL,
        stderr=subprocess.DEVNULL,
        close_fds=True,  # Detach from Typer
    )


def set_window_icon(
    window_title: str,
    icon_path: str | Path,
    cx: int = 0,
    cy: int = 0,
):
    """
    Set the icon of a window given its title.

    :param window_title: The title of the window.
    :param icon_path: Path to the icon file.
    :param cx: Width position of the icon.
    :param cy: Height position of the icon.

    :raises RuntimeError: If an error occurs while setting the window icon.
    """
    if ctypes is None:
        raise RuntimeError(f"ctypes is not available on this platform ({platform.system()}).")

    hwnd = _get_window_hwnd(window_title)

    hicon = ctypes.windll.user32.LoadImageW(
        0,  # hInstance
        str(icon_path),  # icon_path
        1,  # IMAGE_ICON
        cx,  # cx
        cy,  # cy
        0x00000010,  # LR_LOADFROMFILE
    )
    if hicon == 0:
        raise RuntimeError(f"user32.LoadImageW: Failed to load icon from '{icon_path}'.")

    ctypes.windll.user32.SendMessageW(hwnd, 0x80, 0, hicon)  # WM_SETICON (small)
    ctypes.windll.user32.SendMessageW(hwnd, 0x80, 1, hicon)  # WM_SETICON (big)


__all__ = [
    "is_admin",
    "reload_user_path",
    "restart_explorer",
    "set_window_icon",
]



# src\file_conversor\utils\dominate_bulma\page_form.py

from typing import Any

# user-provided modules
from file_conversor.utils.bulma_utils.general import OverwriteFilesField

from file_conversor.utils.dominate_bulma.form_button import Button
from file_conversor.utils.dominate_bulma.form import Form
from file_conversor.utils.dominate_bulma.breadcrumb import Breadcrumb
from file_conversor.utils.dominate_bulma.page_base import PageBase

from file_conversor.utils.dominate_utils import *

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def PageForm(
        *form_fields,
        api_endpoint: str,
        nav_items: list[dict[str, Any]],
        _title: str,
        **kwargs: Any,
):
    """
    Create a page with a form.

    ```python
    nav_items = [{
        'icon': (Optional) The icon name to display alongside the label
        'label': The display text for the item,
        'url': The URL the item links to,
        'active': A boolean indicating if the item is the current page,
    }]
    ```

    :param form_fields: The fields to include in the form.
    :param api_endpoint: The API endpoint to submit the form to.
    :param nav_items: The breadcrumb navigation items.
    :param _title: The title of the page.
    """
    return PageBase(
        Form(
            *form_fields,
            OverwriteFilesField(),
            Button(
                _("Execute"),
                _class="is-primary",
                _title=_("Execute operation"),
                **{
                    ':class': """{
                        'is-loading': checkLoading(),
                        'is-cursor-progress': checkLoading(),
                    }""",
                    ':disabled': '!isValid || checkLoading()',
                    '@click': f'$store.form.submit($el.closest("form[x-data]"), `{api_endpoint}`)',
                },
            ),
            _class='is-flex is-flex-direction-column is-align-items-flex-end is-full-width',
            **{'x-data': r"""{ 
                    isValid: false, 
                    elements: [],                    
                    checkLoading() {
                        return this.$store.status_bar.started && !this.$store.status_bar.finished;
                    },
                    updateValidity() {
                        let res = true;     
                        if (!this.elements || this.elements.length === 0) {   
                            this.elements = this.$el.querySelectorAll('.field[x-data]');
                        }
                        this.elements.forEach((item) => {
                            if (item === this.$el) return;
                            const data = Alpine.$data(item);
                            const valid = data.isValid;
                            if (valid === undefined) return;
                            if (!valid) {
                                res = false;
                            }
                        });
                        this.isValid = res;
                    },
                    init() {
                        this.updateValidity();
                    },  
               }"""},
            **kwargs,
        ),
        _title=_title,
        nav_title=Breadcrumb(*nav_items),
    )


__all__ = [
    "PageForm",
]



# src/file_conversor/backend/gui/image/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui.image._index import image_index

from file_conversor.backend.gui.image.antialias import image_antialias
from file_conversor.backend.gui.image.blur import image_blur
from file_conversor.backend.gui.image.compress import image_compress
from file_conversor.backend.gui.image.convert import image_convert
from file_conversor.backend.gui.image.enhance import image_enhance
from file_conversor.backend.gui.image.filter import image_filter
from file_conversor.backend.gui.image.info import image_info
from file_conversor.backend.gui.image.mirror import image_mirror
from file_conversor.backend.gui.image.render import image_render
from file_conversor.backend.gui.image.resize import image_resize
from file_conversor.backend.gui.image.rotate import image_rotate
from file_conversor.backend.gui.image.to_pdf import image_to_pdf
from file_conversor.backend.gui.image.unsharp import image_unsharp


def routes():
    return [
        FlaskRoute(
            rule="/image",
            handler=image_index,
        ),
        FlaskRoute(
            rule="/image/convert",
            handler=image_convert,
        ),
        FlaskRoute(
            rule="/image/render",
            handler=image_render,
        ),
        FlaskRoute(
            rule="/image/to_pdf",
            handler=image_to_pdf,
        ),
        FlaskRoute(
            rule="/image/compress",
            handler=image_compress,
        ),
        FlaskRoute(
            rule="/image/mirror",
            handler=image_mirror,
        ),
        FlaskRoute(
            rule="/image/rotate",
            handler=image_rotate,
        ),
        FlaskRoute(
            rule="/image/resize",
            handler=image_resize,
        ),
        FlaskRoute(
            rule="/image/antialias",
            handler=image_antialias,
        ),
        FlaskRoute(
            rule="/image/blur",
            handler=image_blur,
        ),
        FlaskRoute(
            rule="/image/enhance",
            handler=image_enhance,
        ),
        FlaskRoute(
            rule="/image/filter",
            handler=image_filter,
        ),
        FlaskRoute(
            rule="/image/unsharp",
            handler=image_unsharp,
        ),
        FlaskRoute(
            rule="/image/info",
            handler=image_info,
        ),
    ]


__all__ = ['routes']



# src/file_conversor/backend/gui/hash/index.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.gui._dom_page import *
from file_conversor.backend.gui.hash._dom_page import *

from file_conversor.utils.dominate_bulma import *

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def hash_index():
    tools = [
        {
            'image': url_for('icons', filename='check.ico'),
            'title': _("Check"),
            'subtitle': _("Checks a hash file (.sha256, .sha1, etc)."),
            'url': url_for('hash_check'),
        },
        {
            'image': url_for('icons', filename='new_file.ico'),
            'title': _("Create"),
            'subtitle': _("Creates a hash file (.sha256, .sha1, etc)."),
            'url': url_for('hash_create'),
        },
    ]
    return render_template_string(str(
        PageCardGrid(
            *tools,
            nav_items=[
                home_nav_item(),
                hash_index_nav_item(active=True),
            ],
            _title=f"{_('Hash Tools')} - File Conversor",
        ))
    )



# src\file_conversor\utils\typer.py

import typer
from typer.models import OptionInfo

from pathlib import Path
from typing import Annotated, Any, Iterable, List

# user-provided modules
from file_conversor.config.locale import get_translation
from file_conversor.utils.validators import check_file_exists, check_dir_exists, check_file_format, check_file_size_format, check_positive_integer, check_valid_options, check_video_resolution

_ = get_translation()


def InputFilesArgument(backend_or_iterable: type | dict | list | None = None):
    list_formats: list[str] | dict[str, Any]
    if not backend_or_iterable:
        list_formats = ["*"]
    elif isinstance(backend_or_iterable, (dict, list)):
        list_formats = backend_or_iterable
    else:
        list_formats = backend_or_iterable.SUPPORTED_IN_FORMATS
    return typer.Argument(
        help=f"{_('Input files')} ({', '.join(list_formats)})",
        callback=lambda x: check_file_format(x, [] if "*" in list_formats else list_formats, exists=True),
    )


def FormatOption(backend_or_iterable: type | dict | list) -> OptionInfo:
    """--format, -f"""
    if isinstance(backend_or_iterable, (dict, list)):
        list_formats: list[str] | dict[str, Any] = backend_or_iterable
    else:
        list_formats: list[str] | dict[str, Any] = backend_or_iterable.SUPPORTED_OUT_FORMATS
    return typer.Option(
        "--format", "-f",
        help=f"{_('Output format')} ({', '.join(list_formats)})",
        callback=lambda x: check_valid_options(x, list_formats),
    )


def OutputDirOption() -> OptionInfo:
    """--output-dir, -od"""
    return typer.Option(
        "--output-dir", "-od",
        help=f"{_('Output directory')}. {_('Defaults to current working directory')}.",
        callback=lambda x: check_dir_exists(x, mkdir=True),
    )


def OutputFileOption(backend: type) -> OptionInfo:
    """--output-file, -of"""
    return typer.Option(
        "--output-file", "-of",
        help=f"{_('Output file')} ({', '.join(backend.SUPPORTED_OUT_FORMATS)}). {_('Defaults to None')} ({_('use the same 1st input file as output name')}).",
        callback=lambda x: check_file_format(x, backend.SUPPORTED_OUT_FORMATS),
    )

#################
# IMAGE OPTIONS #
#################


def QualityOption(prompt: bool | str = False) -> OptionInfo:
    """--quality, -q"""
    return typer.Option(
        "--quality", "-q",
        help=_("Image quality. Valid values are between 1-100."),
        prompt=prompt,
        min=1, max=100,
    )


def AxisOption(prompt: bool | str = False) -> OptionInfo:
    """--axis, -a"""
    return typer.Option(
        "--axis", "-a",
        help=_("Mirror axis. Valid values are 'x' (mirror horizontally) or 'y' (flip vertically)."),
        prompt=prompt,
        callback=lambda x: check_valid_options(x, valid_options=['x', 'y']),
    )


def DPIOption(prompt: bool | str = False) -> OptionInfo:
    """--dpi, -d"""
    return typer.Option(
        "--dpi", "-d",
        help=_("Image quality in dots per inch (DPI). Valid values are between 40-3600."),
        prompt=prompt,
        min=40, max=3600,
    )


def BrightnessOption(prompt: bool | str = False) -> OptionInfo:
    """--brightness, -b"""
    return typer.Option(
        "--brightness", "-b",
        prompt=prompt,
        help=_("Adjust brightness. brightness = 1.00 means no change. brightness < 1.00 makes image black. brightness > 1.00 makes image lighter."),
    )


def ContrastOption(prompt: bool | str = False) -> OptionInfo:
    """--contrast, -ct"""
    return typer.Option(
        "--contrast", "-ct",
        prompt=prompt,
        help=_("Adjust contrast. contrast = 1.00 means no change. contrast < 1.00 reduces contrast (grayish image). contrast > 1.00 increases contrast."),
    )


def ColorOption(prompt: bool | str = False) -> OptionInfo:
    """--color, -cl"""
    return typer.Option(
        "--color", "-cl",
        prompt=prompt,
        help=_("Adjust color. color = 1.00 means no change. color < 1.00 reduces color saturation. color > 1.00 increases color saturation."),
    )


def SharpnessOption(prompt: bool | str = False) -> OptionInfo:
    """--sharpness, -s"""
    return typer.Option(
        "--sharpness", "-s",
        prompt=prompt,
        help=_("Adjust sharpness. sharpness = 1.00 means no change. sharpness < 1.00 makes image more blurry. sharpness > 1.00 increases image crispness (and noise as well)."),
    )


def RadiusOption(prompt: bool | str = False) -> OptionInfo:
    """--radius, -r"""
    return typer.Option(
        "--radius", "-r",
        help=f'{_("Box radius (in pixels).")}',
        prompt=prompt,
        min=1,
    )

#################
# PDF OPTIONS   #
#################


def PasswordOption() -> OptionInfo:
    """--password, -p"""
    return typer.Option(
        "--password", "-p",
        help=_("Password used to open protected file. Defaults to None (do not decrypt)."),
    )

#################
# VIDEO OPTIONS #
#################


def TargetFileSizeOption(prompt: bool | str = False) -> OptionInfo:
    """--target-size, -ts"""
    return typer.Option(
        "--target-size", "-ts",
        help=f"{_("Target file size.")} {_('Format "size[K|M|G]". If 0, do not limit output file size (use encoding speed and quality options to calculate output file size).')}",
        prompt=prompt,
        callback=lambda x: check_file_size_format(x),
    )


def AudioBitrateOption(prompt: bool | str = False) -> OptionInfo:
    """--audio-bitrate, -ab"""
    return typer.Option(
        "--audio-bitrate", "-ab",
        help=f"{_("Audio bitrate in kbps.")} {_('If 0, let FFmpeg decide best bitrate.')}",
        prompt=prompt,
        callback=lambda x: check_positive_integer(x, allow_zero=True),
    )


def VideoBitrateOption(prompt: bool | str = False) -> OptionInfo:
    """--video-bitrate, -vb"""
    return typer.Option(
        "--video-bitrate", "-vb",
        help=f"{_("Video bitrate in kbps.")} {_('Overrides video quality setting (if set). If 0, use video quality setting to encode video using variable bitrate (CRF) encoding (if supported by codec, otherwise use default bitrate for codec/container)')}.",
        prompt=prompt,
        callback=lambda x: check_positive_integer(x, allow_zero=True),
    )


def AudioCodecOption(available_options: Iterable[str], prompt: bool | str = False) -> OptionInfo:
    """--audio-codec, -ac"""
    return typer.Option(
        "--audio-codec", "-ac",
        help=f'{_("Audio codec. Not all codecs are supported in a given format container (run `file_conversor video list-formats` for more information). Available options are:")} {", ".join(available_options)}. {_('Defaults to None (use the default for the file container)')}.',
        prompt=prompt,
        callback=lambda x: check_valid_options(x, available_options),
    )


def VideoCodecOption(available_options: Iterable[str], prompt: bool | str = False) -> OptionInfo:
    """--video-codec, -vc"""
    return typer.Option(
        "--video-codec", "-vc",
        help=f'{_("Video codec. Not all codecs are supported in a given format container (run `file_conversor video list-formats` for more information). Available options are:")} {", ".join(available_options)}. {_('Defaults to None (use the default for the file container)')}.',
        prompt=prompt,
        callback=lambda x: check_valid_options(x, available_options),
    )


def VideoEncodingSpeedOption(options: list[str], prompt: bool | str = False) -> OptionInfo:
    """--video-encoding-speed, -ves"""
    return typer.Option(
        "--video-encoding-speed", "-ves",
        help=f'{_("Video encoding speed/quality trade-off. Available options are:")} {", ".join(options)}. {_("Faster encoding speed usually results in lower video quality and larger file size")}. {_("Defaults to 'medium'")}.',
        prompt=prompt,
        callback=lambda x: check_valid_options(x, options),
    )


def VideoQualityOption(options: list[str], prompt: bool | str = False) -> OptionInfo:
    """--video-quality, -vq"""
    return typer.Option(
        "--video-quality", "-vq",
        help=f'{_("Video quality preset. Available options are:")} {", ".join(options)}. {_("Higher quality usually results in larger file size. Video bitrate (if set) overrides this setting.")}. {_("Defaults to 'medium'")}.',
        prompt=prompt,
        callback=lambda x: check_valid_options(x, options),
    )


def ResolutionOption(prompt: bool | str = False) -> OptionInfo:
    """--resolution, -rs"""
    return typer.Option(
        "--resolution", "-rs",
        help=f'{_("Video target resolution. Format WIDTH:HEIGHT (in pixels). Defaults to None (use same resolution as video source)")}',
        prompt=prompt,
        callback=check_video_resolution,
    )


def FPSOption(prompt: bool | str = False) -> OptionInfo:
    """--fps, -fp"""
    return typer.Option(
        "--fps", "-fp",
        help=f'{_("Target video FPS (frames per second). Uses ``minterpolate`` filter to adjust fps. Defaults to None (use same fps as video source)")}',
        prompt=prompt,
        min=1,
    )


def VideoRotationOption(prompt: bool | str = False) -> OptionInfo:
    """--rotation, -r"""
    return typer.Option(
        "--rotation", "-r",
        help=f'{_("Rotate video (clockwise). Available options are:")} {", ".join(['-180', '-90', '90', '180'])}. Defaults to None (do not rotate).',
        prompt=prompt,
        callback=lambda x: check_valid_options(x, [-180, -90, 90, 180]),
    )


def GammaOption(prompt: bool | str = False) -> OptionInfo:
    """--gamma, -g"""
    return typer.Option(
        "--gamma", "-g",
        prompt=prompt,
        help=f'{_("Adjust gamma")}. {_('gamma = 1.00 means no change. gamma < 1.00 makes image more darker. gamma > 1.00 increases image lightenss')}.',
    )


def UnsharpOption() -> OptionInfo:
    """--unsharp, -u"""
    return typer.Option(
        "--unsharp", "-u",
        help=f'{_("Increase video sharpness (using unsharp mask). May increase image noise.")}. {_('Defaults to False (do not apply filter)')}.',
        is_flag=True,
    )


def DeshakeOption() -> OptionInfo:
    """--deshake, -d"""
    return typer.Option(
        "--deshake", "-d",
        help=f'{_("Deshake video (attempt to fix vertical/horizontal span from handrecoding)")}. {_('Defaults to False (do not apply filter)')}.',
        is_flag=True,
    )




# src\file_conversor\cli\audio\check_cmd.py

import typer

from rich import print

from typing import Annotated, Any, Callable, List
from pathlib import Path

# user-provided modules
from file_conversor.backend import FFprobeBackend

from file_conversor.cli.audio._typer import COMMAND_NAME, CHECK_NAME

from file_conversor.config import Environment, Configuration, State, Log, get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.backend import FFprobeParser
from file_conversor.utils.typer_utils import InputFilesArgument

from file_conversor.system.win import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = FFprobeBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    # FFMPEG commands
    icons_folder_path = Environment.get_icons_folder()
    for ext in FFprobeBackend.SUPPORTED_IN_AUDIO_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="check",
                description="Check",
                command=f'cmd /k "{Environment.get_executable()} "{COMMAND_NAME}" "{CHECK_NAME}" "%1""',
                icon=str(icons_folder_path / 'check.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_audio_check_cmd(
    input_files: List[Path],
    progress_callback: Callable[[float], Any] = lambda p: p,
) -> None:
    """Execute audio check command."""
    # init backend
    backend = FFprobeBackend(
        install_deps=CONFIG['install-deps'],
        verbose=STATE["verbose"],
    )

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        # display current progress
        parser = FFprobeParser(backend, input_file)
        parser.run()
        progress_callback(progress_mgr.complete_step())

    cmd_mgr = CommandManager(input_files, output_dir=Path(), overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback)

    logger.info(f"{_('FFMpeg check')}: [green][bold]{_('SUCCESS')}[/bold][/green]")


@typer_cmd.command(
    name=CHECK_NAME,
    help=f"""
        {_('Checks audio files for corruption / inconsistencies.')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {CHECK_NAME} input_file.mp3`
    """)
def check(
    input_files: Annotated[List[Path], InputFilesArgument(FFprobeBackend.SUPPORTED_IN_AUDIO_FORMATS)],
):
    execute_audio_check_cmd(input_files)



# src/file_conversor/backend/gui/_components/text.py

from file_conversor.utils.dominate_utils import *


def Title(text: str, **kwargs):
    """
    Create a title element.

    :param text: The title text.
    """
    return p(text, _class="title", **kwargs)


def Content(**kwargs):
    """
    Create a content div.
    """
    return div(_class="content", **kwargs)


__all__ = [
    'Content',
]



# src\file_conversor\cli\audio\_typer.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

# command
COMMAND_NAME = "audio"

# SUBCOMMANDS
CHECK_NAME = "check"
CONVERT_NAME = "convert"
INFO_NAME = "info"



# src/file_conversor/backend/gui/ppt/convert.py

from flask import render_template, render_template_string, url_for
from typing import Any

# user-provided modules
from file_conversor.backend.office import PPT_BACKEND

from file_conversor.backend.gui._dom_page import *
from file_conversor.backend.gui.ppt._dom_page import *

from file_conversor.utils.bulma_utils import *
from file_conversor.utils.dominate_bulma import *

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def PageConvert():
    return PageForm(
        InputFilesField(*[
            f for f in PPT_BACKEND.SUPPORTED_IN_FORMATS],
            description=_("Presentation files")
        ),
        FileFormatField(*[
            (f, f.upper())
            for f in PPT_BACKEND.SUPPORTED_OUT_FORMATS
        ], current_value="pdf"),
        OutputDirField(),
        api_endpoint=f"{url_for('api_ppt_convert')}",
        nav_items=[
            home_nav_item(),
            ppt_index_nav_item(),
            ppt_convert_nav_item(active=True),
        ],
        _title=_("Convert PPT - File Conversor"),
    )


def ppt_convert():
    return render_template_string(str(
        PageConvert()
    ))




# src\file_conversor\cli\text\convert_cmd.py

import typer

from typing import Annotated, Any, Callable, List
from pathlib import Path

from rich import print


# user-provided modules
from file_conversor.backend import TextBackend

from file_conversor.cli.text._typer import COMMAND_NAME, CONVERT_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import FormatOption, InputFilesArgument, OutputDirOption


# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = TextBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    for ext in TextBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="to_xml",
                description="To XML",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" "%1" -f "xml"',
                icon=str(icons_folder_path / 'xml.ico'),
            ),
            WinContextCommand(
                name="to_json",
                description="To JSON",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" "%1" -f "json"',
                icon=str(icons_folder_path / 'json.ico'),
            ),
            WinContextCommand(
                name="to_yaml",
                description="To YAML",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" "%1" -f "yaml"',
                icon=str(icons_folder_path / 'yaml.ico'),
            ),
            WinContextCommand(
                name="to_toml",
                description="To TOML",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" "%1" -f "toml"',
                icon=str(icons_folder_path / 'toml.ico'),
            ),
            WinContextCommand(
                name="to_ini",
                description="To INI",
                command=f'{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" "%1" -f "ini"',
                icon=str(icons_folder_path / 'ini.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_text_convert_cmd(
    input_files: List[Path],
    format: str,
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    text_backend = TextBackend(verbose=STATE["verbose"])

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        text_backend.convert(
            input_file=input_file,
            output_file=output_file,
        )
        progress_callback(progress_mgr.complete_step())

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_suffix=f".{format}")
    logger.info(f"{_('File conversion')}: [bold green]{_('SUCCESS')}[/].")


# text convert
@typer_cmd.command(
    name=CONVERT_NAME,
    help=f"""
        {_('Converts text file formats (json, xml, yaml, etc).')}        
    """,
    epilog=f"""
**{_('Examples')}:** 

- `file_conversor {COMMAND_NAME} {CONVERT_NAME} file1.json -f xml` 
""")
def convert(
    input_files: Annotated[List[Path], InputFilesArgument(TextBackend)],
    format: Annotated[str, FormatOption(TextBackend)],
    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_text_convert_cmd(
        input_files=input_files,
        format=format,
        output_dir=output_dir,
    )



# src\file_conversor\cli\image\_typer.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

CONVERSION_PANEL = _("Conversions")
TRANSFORMATION_PANEL = _("Transformations")
FILTER_PANEL = _("Filters")
OTHERS_PANEL = _("Other commands")

# command
COMMAND_NAME = "image"

# SUBCOMMANDS
ANTIALIAS_NAME = "antialias"
BLUR_NAME = "blur"
COMPRESS_NAME = "compress"
CONVERT_NAME = "convert"
ENHANCE_NAME = "enhance"
FILTER_NAME = "filter"
INFO_NAME = "info"
MIRROR_NAME = "mirror"
RENDER_NAME = "render"
RESIZE_NAME = "resize"
ROTATE_NAME = "rotate"
TO_PDF_NAME = "to-pdf"
UNSHARP_NAME = "unsharp"



# src\file_conversor\utils\progress_manager.py


# user-provided
from file_conversor.utils.rich_utils import get_progress_bar

from file_conversor.config.locale import get_translation

_ = get_translation()


class ProgressManager:
    def __init__(self, total_out_files: int = 1, total_steps_per_file: int = 1):
        """
        Inits progress manager

        :param total_out_files: Number of output files
        :param total_steps_per_file: Number of processing steps per file
        """
        super().__init__()
        if total_out_files < 1:
            raise ValueError("total_out_files must be >= 1")
        if total_steps_per_file < 1:
            raise ValueError("total_steps_per_file must be >= 1")

        self._progress = get_progress_bar()
        self._total_out_files = total_out_files
        self._total_steps_per_file = total_steps_per_file

        self._completed_files = 0
        self._current_step = 1

        self._file_progress = 100.0 / total_out_files
        self._step_progress = self._file_progress / total_steps_per_file

    @property
    def completed_files(self) -> int:
        return self._completed_files

    @completed_files.setter
    def completed_files(self, value: int):
        if value > self._total_out_files:
            raise RuntimeError("Completed files > Total out files")
        self._completed_files = value

    @property
    def current_step(self) -> int:
        return self._current_step

    @current_step.setter
    def current_step(self, value: int):
        if value > self._total_steps_per_file:
            self.completed_files += 1
            self._current_step = 1
            return
        self._current_step = value

    def __enter__(self):
        self._progress.__enter__()
        self._task = self._progress.add_task(_("Processing files:"), total=100.0)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._progress.__exit__(exc_type, exc_val, exc_tb)

    def update_progress(self, step_progress: float) -> float:
        """Update progress for current file"""
        # previous completed files
        total_progress = self._completed_files * self._file_progress
        # previous completed steps (of current file)
        total_progress += (self._current_step - 1) * self._step_progress
        # current step (of current file)
        total_progress += (step_progress / 100.0) * self._step_progress
        self._progress.update(self._task, completed=total_progress)
        return total_progress

    def complete_step(self) -> float:
        """Mark current step as completed and move to next step"""
        progress = self.update_progress(100.0)  # Ensure current step is 100%
        self.current_step += 1
        return progress



# src\file_conversor\cli\config\_typer.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

COMMAND_NAME = "config"

# SUBCOMMANDS
SET_NAME = "set"
SHOW_NAME = "show"
